/***************************************************************************
                          basisgraph.cpp  -  description
                             -------------------
    begin                : Thu June 29 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "basisgraph.h"

#include "nodedata.h"
#include "nodeid.h"

#include "edgedata.h"
#include "edgeid.h"

#include "utilities.h"
#include "trace.h"

#include "matrix.h"

#include <algorithm>

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
BasisGraph::BasisGraph(int n) : number_(n),
                                nodes(list<Node>()),
                                edges(list<Edge>())

{ ; }

BasisGraph::BasisGraph(const BasisGraph& basis) :
                                          number_(basis.number_),
                                          nodes(basis.nodes),
                                          edges(basis.edges)
{ ; }

//---------------------------------------------------------------------------
// Operators
//---------------------------------------------------------------------------
BasisGraph& BasisGraph::operator= (const BasisGraph& g)
{
  if (this == &g) return *this;
  number_ = g.number_;
  nodes = g.nodes;
  edges = g.edges;
  return *this;
}

//---------------------------------------------------------------------------
// I/O
//---------------------------------------------------------------------------
ostream& operator<< (ostream& os, const BasisGraph& g)
{
  constNodeIter i_n;
  constEdgeIter i_e;

  os << "graph [" << endl;
  os << "  number " << g.index() << endl;

  if (!g.nodes.empty())
  {
    for (i_n=g.nodes.begin();i_n!=g.nodes.end();i_n++)
    {
      os << "  node [\n    id " << i_n->id();
      os << "\n    label \"" << i_n->label() << "\"";
      os << "\n    graphics [";
      os.precision(4);
      os.setf(ios::fixed,ios::floatfield);
      os << "\n      x " << i_n->coord().x;
      os << "\n      y " << i_n->coord().y;
      os << "\n      z " << i_n->coord().z;
      os << "\n    ]";
      os << "\n  ]" << endl;
    }
  }

  if (!g.edges.empty())
  {
    for(i_e=g.edges.begin();i_e!=g.edges.end();i_e++)
    {
        os << "  edge [\n    source " << i_e->source().id();
        os << "\n    target " << i_e->target().id() ;
        if (i_e->label() != "")
          os << "\n    label \"" << i_e->label() << "\"";
        os << "\n  ]" << endl;
    }
  }

  os << "]" << endl;
  return os;
}

ostream& operator<< (ostream& os, const BasisGraph* g)
{
  constNodeIter i_n;
  constEdgeIter i_e;

  os << "  number " << g->index() << endl;

  if (!g->nodes.empty())
  {
    os << "nodes:\t";
    for (i_n=g->nodes.begin();i_n!=g->nodes.end();i_n++)
      os << "\t"<< *i_n << endl;
  }

  if (!g->edges.empty())
  {
    os << "edges:\t";
    for(i_e=g->edges.begin();i_e!=g->edges.end();i_e++)
        os << "\t" << *i_e << endl;
  }

  os << endl;
  return os;
}
//---------------------------------------------------------------------------
// Information
//---------------------------------------------------------------------------
Matrix* BasisGraph::makeAdjacency() const
{
  unsigned size = nodes.size();
  if (size == 0)
    THROW(EmptyGraphError,"Matrix BasisGraph::makeAdjacency() const")

  // initialize map Node<->unsigned
  initNodeNumberMap();

  double *tmp = new double[size*size];
  fill(tmp,tmp+size*size,0);

  constEdgeIter it  = edges.begin();
  constEdgeIter end = edges.end();
  unsigned source, target;

  // iterate through all nodes of the graph
  while (it != end)
  {
    source = nodeMap.find(it->source())->second;
    target = nodeMap.find(it->target())->second;

    // source -> target
    tmp[source + target*size] = 1;
    // target -> source
    tmp[target + source*size] = 1;
    it++;
  }

  return (new Matrix(size,tmp));
}

Matrix* BasisGraph::makeDegree() const
{
  unsigned size = nodes.size();
  if (size == 0)
    THROW(EmptyGraphError,"Matrix BasisGraph::makeAdjacency() const")

  // initialize map Node<->unsigned
  if (nodeMap.empty())
    initNodeNumberMap();

  double *tmp = new double[size*size];
  fill(tmp,tmp+size*size,0);

  constNodeIter it = nodes.begin();
  constNodeIter end= nodes.end();
  unsigned  number = 0;

  while (it != end)
  {
    number = nodeMap.find(*it)->second;
    tmp[number + number*size] = it->degree();
    it++;
  }

  return (new Matrix(size,tmp));
}

//---------------------------------------------------------------------------
// Search
//---------------------------------------------------------------------------
constNodeIter BasisGraph::findNode(unsigned id) const
{
  return find_if(nodes.begin(),nodes.end(),NodeId(id));
}
NodeIter BasisGraph::findNode(unsigned id)
{
  return find_if(nodes.begin(),nodes.end(),NodeId(id));
}

NodeIter BasisGraph::findGreaterNode(unsigned id)
{
  return find_if(nodes.begin(),nodes.end(),NodeGreaterId(id));
}

constEdgeIter BasisGraph::findEdge(unsigned s, unsigned t) const
{
  return find_if(edges.begin(),edges.end(),EdgeSourceTarget(s,t));
}
EdgeIter BasisGraph::findEdge(unsigned s, unsigned t)
{
  return find_if(edges.begin(),edges.end(),EdgeSourceTarget(s,t));
}

//---------------------------------------------------------------------------
// Deletion
//---------------------------------------------------------------------------
void BasisGraph::clear()
{
  edges.clear();
  nodes.clear();
  nodeMap.clear();
}

//---------------------------------------------------------------------------
// Private Functions
//---------------------------------------------------------------------------
void BasisGraph::initNodeNumberMap() const
{
  if (!nodeMap.empty())
    nodeMap.clear();

  constNodeIter it  = nodes.begin();
  constNodeIter end = nodes.end();
  unsigned  number = 0;

  // iterate through all nodes of the graph
  while (it != end)
    nodeMap.insert(nodeMap.end(),make_pair(*it++,number++));
}





} // namespace graphlib

/* end of file */
