/***************************************************************************
                          bicomponents.cpp  -  description
                             -------------------
    begin                : Thu Mar 8 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "bicomponents.h"

#include "nodedata.h"
#include "nodeid.h"

#include "edgedata.h"
#include "edgeid.h"

#include "subgraphsize.h"
#include "samecomponent.h"

#include "trace.h"
#include "utilities.h"

#include <algorithm>

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
BiComponents::BiComponents(const Graph* g)
  throw(EmptyGraphError,logic_error,EmptyNodeError)
                                           : graph(g),
                                             bicomp(list<Component*>()),
                                             comp(list<Component*>()),
                                             nodeMap(NodeMap()),
                                             edgeMap(EdgeMap())
{
  if (graph->numberNodes()==0)
    THROW(EmptyGraphError,
      "BiComponents::BiComponents(const Graph&): empty node set")
  if (graph->numberEdges()==0)
    THROW(EmptyGraphError,
      "BiComponents::BiComponents(const Graph&): empty edge set")

    // creates the map <Node,Data for Algorithm>
  makeNodeMap();

  // creates the map <Edge,Data for Algorithm>
  makeEdgeMap();

  // biconnectivity algorithm
  graph2BiconnComp();

  // write components
  #ifdef GRAPHLIB_DEBUG
    ofstream out("components.graphlib");
    writeCompBiCompNodes(out);
  #endif

  // sort components by length (biggest first)
  comp.sort(SubGraphSize());
  // sort biconnected components by the component they belong to and the length
  bicomp.sort(SameComponent());
}

BiComponents::~BiComponents()
{
  list<Component*>::iterator it = comp.begin();
  list<Component*>::iterator end = comp.end();

  // deleting connected components
  while (it != end)
    if (*it) delete *it++;

  // deleting biconnected components
  it = bicomp.begin();
  end = bicomp.end();

  while (it != end)
    if (*it) delete *it++;
}

//---------------------------------------------------------------------------
// Output
//---------------------------------------------------------------------------
ostream& operator<<(ostream& os, const BiComponents& bC)
{
  list<Component*>::const_iterator g;

  os << "################### connected components ###################\n";
  for (g=bC.comp.begin(); g!=bC.comp.end(); g++)
    os << **g << endl << endl;

  os << "################### biconnected components ###################\n";
  for (g=bC.bicomp.begin(); g!=bC.bicomp.end(); g++)
    os << **g << endl << endl;

  return os;
}

void BiComponents::map_ausgabe(ostream& os) const
{
  os << endl << "NodeMap" << endl;
  NodeMap::const_iterator it = nodeMap.begin();
  for (;it != nodeMap.end(); it++)
    os << it->first.id() << "\t" << it->second << endl;

  os << endl << "NodeSet" << endl;
  NodeSet::const_iterator its = nodeSet.begin();
  for (;its != nodeSet.end(); its++)
    os << its->id() << "\t" << endl;

  os << endl << "EdgeMap" << endl;
  EdgeMap::const_iterator iter = edgeMap.begin();
  for (;iter != edgeMap.end(); iter++)
    os << iter->first << "\t\t"
       << (iter->second ? "visited" : "not visited") << endl;
}

void BiComponents::writeComponentsNodes(ostream& os) const
{
  list<Component*>::const_iterator it;

  os << "################### connected components ###################\n";
  for (it=comp.begin(); it!=comp.end(); it++)
  {
    copy((*it)->nodesBegin(),(*it)->nodesEnd(),ostream_iterator<Node>(os," "));
    os << endl << endl;
  }

  os << "################### biconnected components ###################\n";
  for (it=bicomp.begin(); it!=bicomp.end(); it++)
  {
    copy((*it)->nodesBegin(),(*it)->nodesEnd(), ostream_iterator<Node>(os," "));
    os << endl << endl;
  }
}

void BiComponents::writeCompBiCompNodes(ostream& os) const
{
  list<Component*>::const_iterator con = comp.begin();
  list<Component*>::const_iterator conEnd = comp.end();
  list<Component*>::const_iterator bi = bicomp.begin();
  list<Component*>::const_iterator biEnd = bicomp.end();

  int conNr=0;

  // nr : N cyclomNr [ nodes ]
  while (con != conEnd)
  {
    conNr = (*con)->index();
    os << "! " << conNr << " : " << (*con)->numberNodes() << " < ";
    os << (*con)->numberEdges()-(*con)->numberNodes()+1 << " > [ ";
    copy((*con)->nodesBegin(),(*con)->nodesEnd(),ostream_iterator<Node>(os," "));
    os << " ]" << endl;

    // all biconnected components belonging to this connected component
    while (bi != biEnd && (*bi)->compNr() == conNr)
    {
      os << "@\t" << (*bi)->index() << " : " << (*bi)->numberNodes() << " < ";
      os << (*bi)->numberEdges()-(*bi)->numberNodes()+1 << " > [ ";
      copy((*bi)->nodesBegin(),(*bi)->nodesEnd(),ostream_iterator<Node>(os," "));
      os << " ]" << endl;
      bi++;
    }
    os << endl;
    con++;
  }
}

//---------------------------------------------------------------------------
// Utilities
//---------------------------------------------------------------------------
void BiComponents::graph2BiconnComp() throw(logic_error,EmptyNodeError)
{
  unsigned step = 1;
  list<Edge> edge_stack;
  list<Edge> conn_edges;
  Node v, w, father;
  constEdgeIter e;
  Component *conn;
  unsigned connNr=1, biNr=1;
  list<Component*>::iterator last_comp_end=bicomp.begin();

  // root of DFS
  v = *(graph->nodesBegin());
  nodeMap[v].setBiCompData(true,step,step);
  nodeSet.erase(v);

  for (;;)
  {
    //
    // single node without any edges
    //
    if (v.degree() == 0)
    {
      list<Node> nodeList;
      nodeList.push_back(v);

      // new component - belongs to NULL
      conn = new Component(graph,nodeList.begin(),nodeList.end(),connNr++);
      comp.push_back(conn);

      // new biconnected component - belonging to the last connected component
      Component *bicon =
        new Component(graph,nodeList.begin(),nodeList.end(),biNr++);
      bicon->compNr(comp.size()+1);
      bicon->belongs2component(conn);
      bicomp.push_back(bicon);

      // set the pointer to the last biconnected component of this component
      last_comp_end++;

      // no nodes left
      if (nodeSet.empty())
        return;

      // new current node
      v = *(nodeSet.begin());
      nodeMap[v].setBiCompData(true,step,step);
      nodeSet.erase(v);
    }
    //
    // forall edges incident to v (current node)
    //
    else if ((e=findUnvisitedEdge(v)) != v.adjEnd())
    {
      // push edge e(v,w) into the stack and mark as visited
      edge_stack.push_back(*e);
      conn_edges.push_back(*e);
      edgeMap[*e] = true;
      w = (v.opposite(*e));

      // w already marked
      if (nodeMap[w].isMarked())
        nodeMap[v].changeLow(min(nodeMap[v].getLow(),nodeMap[w].getDNF()));
      // w not marked
      else
      {
        step++;
        nodeMap[w].setBiCompData(true,step,step,v);
        v = w;
        nodeSet.erase(v);
      }
    }

    //
    // all edges of v visited
    //
    else
    {
      // father(v)
      father=nodeMap[v].getFather();

      // v = root of DFS
      if (father.isNULL())
      {
        if (!edge_stack.empty())
          THROW(logic_error,
            "BiComponents::Graph2BiconnComp(const Graph&) stack not empty")

        // all nodes marked (last component) -> End
        if (nodeSet.empty())
        {
          // connected component - all visited edges
          conn = new Component(graph,conn_edges.begin(),conn_edges.end(),connNr++);
          setComponents(conn, ++last_comp_end, bicomp.end());
          comp.push_back(conn);
          conn_edges.clear();
          return;  // END
        }

        // another component
        else
        {
          // connected component - all visited edges
          conn = new Component(graph,conn_edges.begin(),conn_edges.end(),connNr++);
          setComponents(conn, ++last_comp_end, bicomp.end());
          comp.push_back(conn);
          conn_edges.clear();

          // set the pointer to the last biconnected component of this component
          last_comp_end = --bicomp.end();

          v = *(nodeSet.begin());
          nodeMap[v].setBiCompData(true,step,step);
          nodeSet.erase(v);
        }
      }

      // there is a father of v
      else
      {
        // found a biconnected component
        if (nodeMap[v].getLow() >= nodeMap[father].getDNF())
        {
          stack2Component(edge_stack, father,v, biNr++);
        }

        //
        nodeMap[father].changeLow(min(nodeMap[v].getLow(),
                                          nodeMap[father].getLow()));

        v = father;
      }
    }
  }
}

void BiComponents::makeNodeMap()
{
  constNodeIter  it  = graph->nodesBegin();
  constNodeIter  end = graph->nodesEnd();

  BiCompData data;

  while (it != end)
  {
    nodeMap.insert(nodeMap.end(),make_pair(*it,data));
    nodeSet.insert(nodeSet.end(),*it);
    it++;
  }
}

void BiComponents::makeEdgeMap()
{
  constEdgeIter it  = graph->edgesBegin();
  constEdgeIter end = graph->edgesEnd();

  while (it != end)
    edgeMap.insert(edgeMap.end(),make_pair(*it++,false));
}

//---------------------------------------------------------------------------
// Internal
//---------------------------------------------------------------------------
constEdgeIter BiComponents::findUnvisitedEdge(const Node& n) const
  throw(EmptyNodeError)
{
  if (n.isNULL())
    THROW(EmptyNodeError,
      "BiComponents::findUnvisitedEdge(const Node&) const: node has no data.")

  constEdgeIter it  = n.adjBegin();
  constEdgeIter end = n.adjEnd();

  while (it != end)
  {
    // if edge unvisited -> return
    if (edgeMap.find(*it)->second == false)
      return it;
    else
      it++;
  }
  return it;
}

void BiComponents::stack2Component(list<Edge>& edge_stack,const Node& father,
                                   const Node& v, unsigned biNr, bool all)
{
  Component *g;

  // take the whole stack
  if (all)
  {
    g = new Component(graph,edge_stack.begin(),edge_stack.end(),biNr);
    edge_stack.clear();
  }

  // take all edges from the edge(father,v) to the end
  else
  {
    list<Edge>::iterator begin;

    // search for the edge (father,v) resp. e(v,father)
    begin = find_if(edge_stack.begin(),edge_stack.end(),
                    EdgeSourceTarget(father.id(),v.id()));

    // make subgraph
    g = new Component(graph,begin,edge_stack.end(),biNr);

    // remove "used" edges
    edge_stack.erase(begin,edge_stack.end());
  }

  // add bicomp to the list
  g->compNr(comp.size()+1);
  bicomp.push_back(g);

}

void BiComponents::setComponents(const Component* conn, list<Component*>::iterator it, list<Component*>::iterator end) {

  while (it != end)
  {
    (*it)->belongs2component(conn);
    it++;
  }


}


} // namespace graphlib

/* end of file */
