/***************************************************************************
                          bitvector.h  -  description                              
                             -------------------                                         
    begin                : Tue Dec 7 1999                                           
    copyright            : (C) 1999 by Gleiss Petra                         
    email                : petra@gleiss.at                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 * pos 0 == the very left position                                         *
 * pos size-1 == the very right position                                   *
* ***************************************************************************/

#include<stdlib.h>
#include<vector>
#include<iostream>
#include<iomanip.h>
#include<stdexcept>
#include<limits.h> // CHAR_BIT = number of bits in char
#include<algorithm>
#include<functional>
#include<time.h>

#include "graphlib_error.h"
using namespace graphlibError;

#ifndef BITVECTOR_H
#define BITVECTOR_H

namespace graphlib {

// type of buffer = word
typedef unsigned long Bit_Vec_Type;
typedef Bit_Vec_Type Bit_Vec;

// bytes of word
const unsigned BYTES = sizeof (Bit_Vec);
// bits of word
const unsigned BITS  = BYTES * CHAR_BIT;
// width of tabulator for setw()
const unsigned TABW = 4;

/**
  * @short BitVector
  *
  * A bitfield for effizient storing of 0/1 informations.
  * Constant access of size in bits of the field, the number
  * of set bits.
  * Position 0 is the very left position.
  * Position (size-1) is the very right position.
  */

class BitVector
{
    //------------------------------------------------------ friend classes
    /**
      * Output operator.
      * Creates following output:
      *  size  length |bitfield(0/1)|
      *
      * @param os the output-stream
      * @param b the bitvector
      * @return the output-stream
      */
    friend ostream& operator<< (ostream& os, const BitVector& b)
          throw(logic_error,BufferError,OutsideError);

    /**
      * Output operator.
      * Creates following output:
      *  bitfield(0/1)
      *
      * @param os the output-stream
      * @param b the pointer to the bitvector
      * @return the output-stream
      */
    friend ostream& operator<< (ostream& os, const BitVector* b)
        throw(logic_error,BufferError,OutsideError);

  public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Constructor.
      * Generates a bitfield of with <code>size</code> bits..
      */
    BitVector(unsigned size=0);

   /**
     * CopyConstructor
     */
    BitVector(const BitVector&);

    /**
      * CopyConstructor
      */
    BitVector(const BitVector*);

    /**
      * Destructor. Deletes buffer
      */
    virtual ~BitVector();

    //------------------------------------------------------ Operators
    /**
      * Assignement operator
      */
    BitVector& operator=(const BitVector&);

    /**
      * Assigns <code>abuffer</code> to <var>buffer</var>.
      * <par>
      * Note: Only the first <var>size</var> "0" and "1" are taken,
      * all other characters and remaining ones are ignored.
      *
      * @param abuffer
      */
    BitVector& operator=(const char* abuffer);

    /**
      * Returns true if the bitfields itself are the same.
      */
    bool operator==(const BitVector&) const throw(SizeError);

    /**
      * Returns true if the bitfields itself are different.
      */
    bool operator!=(const BitVector&) const throw(SizeError);

    /**
      * Returns true if <code>this</code> has less bits set than <code>other</code>.
      */
    bool operator<(const BitVector& other) const throw(SizeError);

    /**
      * Returns true if <code>this</code> has more bits set than <code>other</code>.
      */
    bool operator>(const BitVector& other) const throw(SizeError);

    /**
      * Returns <code>this</code> AND <code>other</code>.
      */
    BitVector operator&(const BitVector& other) const throw(SizeError);

    /**
      * Returns <code>this</code> OR <code>other</code>.
      */
    BitVector operator|(const BitVector& other) const throw(SizeError);

    /**
      * Returns <code>this</code> XOR <code>other</code>.
      */
    BitVector operator^(const BitVector& other) const throw(SizeError);

    /**
      * Assigns <code>this</code> AND <code>other</code>.
      */
    BitVector& operator&=(const BitVector& other) throw(SizeError);

    /**
      * Assigns <code>this</code> OR <code>other</code>.
      */
    BitVector& operator|=(const BitVector& other) throw(SizeError);

    /**
      * Assigns <code>this</code> XOR <code>other</code>.
      */
    BitVector& operator^=(const BitVector& other) throw(SizeError);

    /**
      * Assigns NOT (<code>this</code> AND <code>other</code>).
      */
    void AND_Negation(const BitVector& other) throw(SizeError);

    /**
      * Returns true, if <code>this</code> has the more left bits set
      */
    bool greaterValue(const BitVector& other) const throw(SizeError);

    //------------------------------------------------------ Information
    /**
      * Returns the size in bits.
      */
    unsigned getsize() const throw(logic_error);

    /**
      * Returns the size in words.
      */
    unsigned getwords() const throw(logic_error);

    /**
      * Returns the bitfield itself.
      */
    Bit_Vec* getbuffer() const;

    /**
      * Calculates the number of set bits.
      */
    void setLength() const throw(BufferError);

    /**
     * Returns the number of set bits. (If necessary new calculated).
     */
    int count() const throw(BufferError);

    /**
      * Returns true if the bit on position <code>pos</code> is set.
      *
      * @param position
      */
    bool getBit(unsigned pos) const throw(BufferError,OutsideError);

    /**
      * Returns true if no bit is set.
      */
    bool empty() const throw(BufferError);

    /**
      * Sets the bit on position <code>pos</code>, if it is not set already
      * and returns true. Otherwise the bit is cleared and false is returned.
      *
      * @param position
      */
    bool setBit(unsigned pos) throw(BufferError,OutsideError);

    /**
      * Sets the bits from position <code>begin</code> to <code>end</code>.
      * The <code>begin</code> bit is set, the <code>end</code> bit not.
      *
      * @param begin
      * @param end
      */
    BitVector& setBit(unsigned begin, unsigned end)
        throw(BufferError,OutsideError);

    /**
      * Assigns <code>abuffer</code> to the bitfield.
      * <par>
      * Note: Only the first <var>size</var> "0" and "1" are taken,
      * all other characters and remaining ones are ignored.
      *
      * See BitVector#operator(const char*);
      */
    BitVector& setBit(const char* abuffer);

    /**
      * Sets all bits.
      */
    BitVector& setAllBits() throw(BufferError,OutsideError);

    /**
      * Clears the bit on position <code>pos</code>.
      *
      * @param position
      */
    BitVector& clearBit(unsigned pos) throw(BufferError,OutsideError);

    /**
      * Clears the bits from position <code>begin</code> to <code>end</code>.
      * The <code>begin</code> bit is cleared, the <code>end</code> bit not.
      *
      * @param begin
      * @param end
      */
    BitVector& clearBit(unsigned begin, unsigned end)
        throw(BufferError,OutsideError);

    /**
      * Clears all bits.
      */
    BitVector& clearAllBits() throw(BufferError);


  protected:
    //------------------------------------------------------ Datas
    /// size in bits
    unsigned size;
    /// size in words
    unsigned words;
    /// number of set bits (if -1 -> new calculations required)
    mutable int length;
    /// bitvector itself
    Bit_Vec *buffer;

    //------------------------------------------------------ Utilities
    /**
      * Returns the size in bytes.
      */
    unsigned calcBytes() const;
    /**
      * Returns the positions of a bit with <code>number</code>
      * in one word.
      *
      * @param number of the bit
      */
    unsigned getoffset(int number) const;

    /**
      * Copies <var>buffer</var> and sets <var>size, words, length</var>.
      */
    void copy(const BitVector&);


}; // class BitVector

} // namespace graphlib

#endif // BITVECTOR_H

/* end of file */
