/***************************************************************************
                          component.h  -  description
                             -------------------
    begin                : Thu June 29 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COMPONENT_H
#define COMPONENT_H

#include "subgraph.h"

#include <string>
#include <list>
#include <iostream>

namespace graphlib {

/**
  *@short A subgraph of an undirected graph
  *
  * A subgraph <var>G'=(V',E')</var> of a graph <var>G=(V,E)</var>
  * consists of a set of nodes <var>V'</var> contained in <var>V</var>
  * and a set of edges <var>E'</var> contained in <var>E</var>.
  * The nodes and edges of the subgraphs are the same as in the graph.
  * The "owner" of the nodes and edges is the graph <var>G</var>.
  * Contains a pointer to the component to which one this belong. If
  * there is no such component the pointer is NULL.
  *
  * @see Edge
  * @see Node
  * @see Graph
  */

class Component : public SubGraph
{

  public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Constructor.
      * Generates an empty SubGraph, without any nodes and any edges,
      * which belongs to <code>graph</code>, with number <code>n</code>
      */
    Component(const Graph* graph=0, int nr=0);

   /**
     * Constructor.
     * Generates a Component from <code>graph</code> induced by vertex set
     * form <code>begin</code> to <code>end</code>.
     *
     * @param graph
     * @param begin - begin of the vertex set of subgraph
     * @param end   - end of the vertex set of subgraph
     * @param nr - number of the subgraph
     */
   Component(const Graph*, constNodeIter begin, constNodeIter end, int nr=0);

   /**
     * Constructor.
     * Generates a Component from <code>graph</code> induced by edge set form
     * <code>begin</code> to <code>end</code>.
     *
     * @param graph
     * @param begin - begin of the edge set of subgraph
     * @param end   - end of the edge set of subgraph
     * @param nr - number of the subgraph
     */
   Component(const Graph*,constEdgeIter begin, constEdgeIter end, int nr=0);

   /**
     * Destructor. Nothing to do.
     */
   virtual ~Component();

    /**
      * Copy Constructor.
      * Copies the list of nodes and edges, but they still belongs to
      * <var>graph</var>.
      */
    Component(const Component&);

    //------------------------------------------------------ Operators
    /**
      * Assignement Operator
      */
    Component& operator=(const Component& g);

  public:
    //------------------------------------------------------ Information
    /**
      * Returns the number of the component.
      */
    int compNr() const { return comp_nr; }

    /**
      * Sets the <code>number</code> of the component
      *
      * @param number
      */
    void compNr(int number) { comp_nr=number; }

    /**
      * Returns the component to which this one belongs, or NULL.
      */
    const Component* belongs2component()const { return component; }

    /**
      * Sets the <code>component</code> to which this one belongs.
      *
      * @param bel_comp
      */
    void belongs2component(const Component* bel_comp) { component = bel_comp; }

  protected:
    //------------------------------------------------------ Name
    /// number of the component
    int               comp_nr;
    /// this component belongs to component
    const Component         *component;

};  // class Component

} // namespace graphlib

#endif // COMPONENT_H

/* end of file */
