/***************************************************************************
                          cyclegraph.cpp  -  description
                             -------------------
    begin                : Mon May 21 2001
    cou=unodes.begin();pyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "cyclegraph.h"
#include "graphnumber.h"

#include "trace.h"
#include "logfile.h"
#include "graphlib_error.h"
using namespace graphlibError;

#include "utilities.h"
#include "graphlib_utilities.h"

#include "prototype.h"
#include "comparisions.h"

#include "bicomponents.h"

#include <algorithm>

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
CycleGraph::CycleGraph() : Graph(),
                           cycleVec(vector<Cycle*>()),
                           uElemVec(vector<Cycle*>()),
                           unodes(vector<Node>()),
                           components(0),
                           bicomponents(0),
                           biComponents(vector<Component*>()),
                           conComponents(vector<Component*>()),
                           unodesComp(vector<list<Node> >()),
                           cycles(list<Cycle*>()),
                           digraphs(list<Graph*>()),
                           unodeDigraphs(list<Graph*>()),
                           calcStat(NO)
{;}

CycleGraph::~CycleGraph()
{
  // delete all calculations
  clearCalculations();
}

void CycleGraph::clear()
{
  // delete all calculations
  clearCalculations();

  // deletes nodes and edges
  Graph::clear();
}

void CycleGraph::deleteCycles()
{
  CycleVIter it = cycleVec.begin();
  CycleVIter end= cycleVec.end();

  while (it != end)
    if (*it) delete *it++;

  cycleVec.clear();
}

void CycleGraph::deleteUElements()
{
  CycleVIter it = uElemVec.begin();
  CycleVIter end= uElemVec.end();

  while (it != end)
    if (*it) delete *it++;

  uElemVec.clear();
}

void CycleGraph::deleteDiGraphs()
{
  GraphIter it = digraphs.begin();
  GraphIter end= digraphs.end();

  while (it != end)
    if (*it) delete *it++;

  digraphs.clear();
}

void CycleGraph::deleteUnodeDiGraphs()
{
  GraphIter it = unodeDigraphs.begin();
  GraphIter end= unodeDigraphs.end();

  while (it != end)
    if (*it) delete *it++;

  unodeDigraphs.clear();
}

void CycleGraph::deleteBiComp()
{
  SubGraphIter it = biComponents.begin();
  SubGraphIter end= biComponents.end();

  while (it != end)
    if (*it) delete *it++;

  biComponents.clear();
  bicomponents = 0;
}

void CycleGraph::deleteComp()
{
  SubGraphIter it = conComponents.begin();
  SubGraphIter end= conComponents.end();

  while (it != end)
    if (*it) delete *it++;

  conComponents.clear();
  components = 0;
}

void CycleGraph::clearCalculations()
{
  // delete cycles
  if (!cycleVec.empty())
    deleteCycles();

  // delete cycles and u-paths
  if (!uElemVec.empty())
    deleteUElements();

  // delete digraphs
  if (!digraphs.empty())
    deleteDiGraphs();

  // delete unodeDigraphs
  if (!unodeDigraphs.empty())
    deleteUnodeDiGraphs();

  // delete biconnected componets
  if (!biComponents.empty())
    deleteBiComp();

  // delete biconnected componets
  if (!conComponents.empty())
    deleteComp();

  // removes u-nodes
  unodes.clear();

  // removes u-nodes comp
  unodesComp.clear();

  // status of calculations -> NO
  calcStat = NO;

  // clears edgeMap
  edgeMap.clear();
}

//---------------------------------------------------------------------------
// Calculations
//---------------------------------------------------------------------------
void CycleGraph::calcComponents()
{
  // already calculated
  if (components != 0)
    return;

  // calculate biconnected components
  BiComponents bC(this);
//  ofstream out("components.cyclegraph");
//  bC.writeCompBiCompNodes(out);

  { // connected components
    list<Component*>::const_iterator it  = bC.conComponentsSubGraphs().begin();
    list<Component*>::const_iterator end = bC.conComponentsSubGraphs().end();

    components   = bC.numberConComp();

    // pushes connected components into class variable
    while (it != end)
    {
      Component* g = new Component(**it++);
      conComponents.push_back(g);
    }
  }

  { // biconnected components
    list<Component*>::const_iterator it  = bC.biComponentsSubGraphs().begin();
    list<Component*>::const_iterator end = bC.biComponentsSubGraphs().end();

    bicomponents = bC.numberBiComp();

    // pushes biconnected components into class variable
    while (it != end)
    {
      Component* g = new Component(**it++);
      biComponents.push_back(g);
    }
  }
}

void CycleGraph::calcCycleBasis(DecompositionsMode flag)
{
  // nothing to do
  if (flag == None)
    return;

  // prototype index
  unsigned long prototypeIndex=1;

  // xchg-class index
  unsigned long interchangeIndex=1;

  // initialize edgeMap, calculated biconnected components
  init();

  constSubGraphIter it = biComponents.begin();
  constSubGraphIter end= biComponents.end();

  while (it != end)
  {
    list<Prototype> prototypes;
    list<Cycle*> mcb;
    list<Cycle*> relevants;

    // calculate cycles containing prototypes and a mcb
    calcCI(**it, prototypes);

    // only if there is any cycle
    if (prototypes.size() != 0)
    {
      // calculate prototypes
      int cyclomatic = (*it)->numberEdges() - (*it)->numberNodes() + 1;
      calcPrototypes(prototypes, prototypeIndex, cyclomatic, flag);

      if (flag >= Relevants)
      {
        // calculate relevants and extracts a mcb
        calcRelevants(prototypes, relevants, mcb);

        // calculates edge-short and unique edge-short cycles
        if (flag >= UniqueShortests)
          calcShortests(relevants);

        // calculates interchangeability classes and sets interclass index
        // also calculates the essential cycles
        if (flag >= InterchangeabilityClasses)
          calcXchgClasses(relevants, interchangeIndex);

        // calculates essential cycles,
        // if interchangeability classes not calculated
        else if (flag == Essentials)
          calcEssentials(relevants, mcb);
      }
    }

    // delete digraphs for this block
    if (!digraphs.empty())
      deleteDiGraphs();

    // next biconnected component
    it++;
  }

  // sorts cycles by length and sets a unique index over all cylces
  // and puts cycles into cycleVec.
  setCycleIndex();
}

void CycleGraph::calcUBasis(DecompositionsMode flag)
{
  // nothing to do
  if (flag == None)
    return;

  if (unodes.empty())
    THROW(logic_error,"CycleGraph::calcUBasis(DecompositionsMode) "
        " no unodes set")

  // prototype index
  unsigned long prototypeIndex=1;

  // xchg-class index
  unsigned long interchangeIndex=1;

  // initialize edgeMap, calculated  components
  init();

  constSubGraphIter it = conComponents.begin();
  constSubGraphIter end= conComponents.end();

  cVecListNodeIter unode = unodesComp.begin();
  cVecListNodeIter unodeE= unodesComp.end();

  // for all connected components and its unodes
  while (it != end && unode != unodeE)
  {
    list<Prototype> prototypes;
    list<Cycle*> mcb;
    list<Cycle*> relevants;

    // if cycles are not calculated jet
    if (cycleVec.empty())
      // calculate cycles containing prototypes and a mcb
      calcCI(**it, prototypes);
    else
      copyCyclePrototypes2UElem(prototypes);

    // calculate U-path between U-nodes of this component
    calcUpathes(**it, *unode, prototypes);

    // calculate prototypes
//    int cyclomatic = (*it)->numberEdges()-(*it)->numberNodes()
//                     + unodesComp.size() ;
    calcPrototypes(prototypes, prototypeIndex, -1, flag);

    if (flag >= Relevants)
    {
//      if (cycleVec.emtpy())
        // calculate relevants and extracts a mcb
        calcRelevants(prototypes, relevants, mcb);
//      else
//        copyCycleRelevants2UElem(prototypes, relevants, mcb);

      // calculates edge-short and unique edge-short cycles
      if (flag >= UniqueShortests)
        calcShortests(relevants);

      // calculates interchangeability classes and sets interclass index
      // also calculates the essential cycles
      if (flag >= InterchangeabilityClasses)
        calcXchgClasses(relevants, interchangeIndex);

      // calculates essential cycles,
      // if interchangeability classes not calculated
      else if (flag == Essentials)
        calcEssentials(relevants, mcb);
    }

    // delete digraphs for this component
    if (!digraphs.empty())
      deleteDiGraphs();

    // delete digraphs for unodes for this component
    if (!unodeDigraphs.empty())
      deleteUnodeDiGraphs();

    // next component
    it++;
  }

  // sorts cycles by length and sets a unique index over
  // all cylces and u-paths and stores them in uElemVec
  setCycleIndex(true);

}

void CycleGraph::setUnodes(string unodefile)
{
  ifstream in;

  // open unodefile
  in.open(unodefile.c_str(),ios::in);
  if (in.fail() || in.bad())
    THROW(runtime_error,"CycleGraph::setUnodes(string) can not open"
      "U-node file:"+unodefile)

  int node;
  constNodeIter n;

  while (!in.eof())
  {
    in >> node;
    if ((n=findNode(node)) != nodes.end())
      if (find(unodes.begin(),unodes.end(),*n) == unodes.end())
        unodes.push_back(*n);
      else
       ;
    else
      THROW(runtime_error,"CycleGraph::setUnodes(const vector<int>&) "
        "graph contains no node with id "+Ttos(node)+"\n")
  }
}

void CycleGraph::setUnodes(const vector<int>& u)
{
  vector<int>::const_iterator it;
  unodes.clear();

  constNodeIter n;

  for (it=u.begin(); it!=u.end(); it++)
  {
    if ((n=findNode(*it)) != nodes.end())
      if (find(unodes.begin(),unodes.end(),*n) == unodes.end())
        unodes.push_back(*n);
      else
       ;
    else
      THROW(runtime_error,"CycleGraph::setUnodes(const vector<int>&) "
        "graph contains no node with id "+Ttos(*it)+"\n")
  }
}

void CycleGraph::makeCycleStatistic(string fname)
{
  // calculate first, if not already
  if (cycles.empty())
    calcCycleBasis();

  CycleVIter it;
  int length;
  int mcb, umcb, sh, unish, ess;
//  int u_mcb, u_umcb, u_sh, u_unish, u_ess;
  vector<int> v_length;
  vector<int> v_mcb, v_umcb, v_sh, v_unish, v_ess;
//  vector<int> vu_mcb, vu_umcb, vu_sh, vu_unish, vu_ess;

  // statistic for the mcb and the relevant,short, unique short and
  //    essential cycles and upathes and additional for upathes
  length = mcb = umcb = sh = unish = ess = 0;
//  u_mcb = u_umcb = u_sh = u_unish = u_ess = 0;

  for (it=cycleVec.begin(), length=(*it)->count(); it!=cycleVec.end(); it++)
  {
    if (length != (*it)->count())
    {
      v_length.push_back(length);
      v_mcb.push_back(mcb);
      v_umcb.push_back(umcb);
      v_sh.push_back(sh);
      v_unish.push_back(unish);
      v_ess.push_back(ess);
//      vu_mcb.push_back(u_mcb);
//      vu_umcb.push_back(u_umcb);
//      vu_sh.push_back(u_sh);
//      vu_unish.push_back(u_unish);
//      vu_ess.push_back(u_ess);
      length = (*it)->count();
      mcb = umcb = sh = unish = ess = 0;
//      u_mcb = u_umcb = u_sh = u_unish = u_ess = 0;
    }

    // if u-path
//    if ((*it)->getmembers(Cycle::UPath))
//    {
//      if ((*it)->getmembers(Cycle::Mcb)) mcb++, u_mcb++;
//      if ((*it)->getmembers(Cycle::Relevant)) umcb++, u_umcb++;
//      if ((*it)->getmembers(Cycle::Shortest)) sh++, u_sh++;
//      if ((*it)->getmembers(Cycle::UniqueShortest)) unish++, u_unish++;
//      if ((*it)->getmembers(Cycle::Essential)) ess++, u_ess++;
//
//    } else {
      if ((*it)->getmembers(Cycle::Mcb)) mcb++;
      if ((*it)->getmembers(Cycle::Relevant)) umcb++;
      if ((*it)->getmembers(Cycle::Shortest)) sh++;
      if ((*it)->getmembers(Cycle::UniqueShortest)) unish++;;
      if ((*it)->getmembers(Cycle::Essential)) ess++;
//    }

  }
  v_length.push_back(length);
  v_mcb.push_back(mcb);
  v_umcb.push_back(umcb);
  v_sh.push_back(sh);
  v_unish.push_back(unish);
  v_ess.push_back(ess);
//  vu_mcb.push_back(u_mcb);
//  vu_umcb.push_back(u_umcb);
//  vu_sh.push_back(u_sh);
//  vu_unish.push_back(u_unish);
//  vu_ess.push_back(u_ess);

  fname += ".cycle_stat";
  ofstream out;
  out.open(fname.c_str(), ios::app);
  out << "######## cycles  #####################" << endl;
  out << v_length << endl << v_mcb << endl << v_umcb << endl;
  out << v_sh << endl << v_unish << endl << v_ess << endl;
//  out << "---------only u-pathes --------------------" << endl;
//  out << v_length << endl << vu_mcb << endl << vu_umcb << endl;
//  out << vu_sh << endl << vu_unish << endl << vu_ess << endl;
  out.close();
}


bool CycleGraph::indepTest(const list<int>& selected)
{
  list<BitVector> help;
  list<int>::const_iterator it;

  // fill all selected cycles into help
  for (it=selected.begin(); it!=selected.end(); it++)
    help.push_back(*(cycleVec[*it]));

  // number of cycles
  int rank = help.size();

  if (rank==gauss(help, help.begin(), 0))
    return true;

  else
    return false;
}

//---------------------------------------------------------------------------
// private functions for calculations
//---------------------------------------------------------------------------
NodePathMap& CycleGraph::dijkstra(Node node, NodePathMap& nodeMap,
                                  NodeSet& nodeSet, bool ordered) const
{
  #if GRAPHLIB_DEBUG==6
    static bool ersteMal=true;
    LogFile dijk;
    if (ersteMal)
    {
      dijk.init("dijkstra.graphlib");
      ersteMal=false;
    }
    else
      dijk.reopen("dijkstra.graphlib");
    #define DIJK(msg) dijk.logging(msg);
  #else
    #define DIJK(msg) ;
  #endif

  constEdgeIter edgeI;
  Node neighb;
  NodePathMap::iterator nodeI = nodeMap.find(node);
  NodePathMap::iterator neighbI;
  unsigned dist = 0;
  int callingNode = node.id();

  DIJK("calling Node:"+Ttos(callingNode)+" order: "+Ttos(ordered))

  for (;;)
  {
    // distance to new nodes
    dist = nodeI->second.distance + 1;

    #if GRAPHLIB_DEBUG==6
      writeNodeMap(nodeMap,dijk.os());
    #endif
    DIJK("\n\n node: "+Ttos(node)+"\nneighbours: ")

    // for all neighbours
    for (edgeI=node.adjBegin(); edgeI!=node.adjEnd(); edgeI++)
    {
      neighb = node.opposite(*edgeI);
      neighbI = nodeMap.find(neighb);
      DIJK(Ttos(neighb)+" ")

      // neighbour is in this biconnected component
      // and not marked (no shortest path was found till now)
      if (neighbI!=nodeMap.end() && ! neighbI->second.marked)
      {
        // there is already a path to the neighbour, cause distance != inf
        if (neighbI->second.distance < INF)
        {
          // a shorter distance is found
          if (nodeI->second.distance > dist)
          {
            // set new distance
            neighbI->second.distance = dist;

            // if order important and
            // neighbour greater or current's path emtpy -> delete path
            if (ordered &&
                (neighb.id()>callingNode || nodeI->second.path.empty()))
              neighbI->second.path.clear();

            // else neighbour's path = currents path + current node
            else
            {
              neighbI->second.path = nodeI->second.path;
              neighbI->second.path.push_back(neighb);
            }
          }

          // same distance
          else if (neighbI->second.distance == dist)
          {
            // if order important and current's path not empty
            if (ordered && !nodeI->second.path.empty())
            {
              // neighbour greater -> delete path
              if (neighb.id()>callingNode)
                neighbI->second.path.clear();

              // else neighbour's path = currents path + current node
              else
              {
                neighbI->second.path = nodeI->second.path;
                neighbI->second.path.push_back(neighb);
              }
            }
          }
        }

        // new node found - there was no path to this node till now
        else
        {
          // set new distance
          neighbI->second.distance = dist;

          // if order important and
          // neighbour greater or current's path emtpy -> delete path
          if (ordered &&
              (neighb.id()>callingNode || nodeI->second.path.empty()))
            neighbI->second.path.clear();

          // else neighbour's path = currents path + current node
          else
          {
            neighbI->second.path = nodeI->second.path;
            neighbI->second.path.push_back(neighb);
          }
        }
      }
    }

    // mark current node
    nodeI->second.marked = true;
    nodeSet.erase(node);

    // new node = node with smallest distance
    if (!nodeSet.empty())
    {
      node = findNodeWithSmallestDistance(nodeMap,nodeSet);
      nodeI = nodeMap.find(node);
    }
    // END
    else
    {
      DIJK("\n Dijkstra Ende *************************\n")
      return nodeMap;
    }
  }
}

void CycleGraph::initNodeMap(const Node& node, const Component& biComp,
                             NodePathMap& nodeMap, NodeSet& nodeSet) const
{
  constNodeIter it=biComp.nodesBegin();
  constNodeIter end=biComp.nodesEnd();

  // distance = INF, marked = false, path = {node}
  NodePath data(node);

  // clears map and set
  nodeMap.clear();
  nodeSet.clear();

  // all nodes of the graph
  while (it != end)
  {
    nodeMap.insert(nodeMap.end(),make_pair(*it,data));
    nodeSet.insert(nodeSet.end(),*it);
    it++;
  }

  // set for calling node distance = 0
  nodeMap[node].distance = 0;
}

Node CycleGraph::findNodeWithSmallestDistance(const NodePathMap& nodeMap,
                              const NodeSet& nodeSet) const
{
  NodeSet::const_iterator it=nodeSet.begin();
  NodeSet::const_iterator end=nodeSet.end();

  Node result=*it;

  // all nodes in the set
  while (it != end)
  {
    if (nodeMap.find(result)->second.distance >
        nodeMap.find(*it)->second.distance)
      result = *it;

    it++;
  }
  return result;
}

void CycleGraph::calcUpathes(const Component& comp, const list<Node>& unodes,
                        list<Prototype>& prototypes)
{
  #if GRAPHLIB_DEBUG==6
    static bool ersteMal=true;
    LogFile upath;
    if (ersteMal)
    {
      upath.init("upath.graphlib");
      ersteMal=false;
    }
    else
      upath.reopen("upath.graphlib");
    #define UPATH(msg) upath.logging(msg);
  #else
    #define UPATH(msg) ;
  #endif

  // for dijkstra
  NodePathMap nodeMapU, nodeMapV;
  NodeSet     nodeSetU, nodeSetV;

  Prototype *b;
  Graph *gU=0, *gV=0;
  unsigned distance;

  // (same names as in the gleiss et al (2001) paper)
  constNodeIter u,v,w;
  NodePathMap::iterator wU, wV, xU, xV, yU, yV;
  constEdgeIter x_edge, y_edge;

  constNodeIter nodesBeg = comp.nodesBegin();
  constNodeIter nodesEnd = comp.nodesEnd();

  constNodeIter unodesEnd = unodes.end();
  constNodeIter unodesBeforEnd = --(unodes.end());

  // for the first u-node
  u=unodes.begin();

  // forall Pair of U-Nodes
  for (; u!=unodesBeforEnd; u++)
  {
    v = u; v++;
    for (; v!=unodesEnd; v++)
    {
      // if u and v are adjacent -> prototype
      if (u->findEdge(*v) != u->adjEnd())
      {
        b = new Prototype(edges.size());
        b->setNodes(INFINITY,u->id(),v->id());
        // calc path (u,v)
        b->calcCycle(edgeMap);
        // add to prototypes
        prototypes.push_back(*b);
        UPATH("\nadjacent:  "+Ttos(*b)+"\n")
        delete b;

        // next v
        continue;
      }

      // digraphs for calculation of relevant u-paths, if not made yet
      if (find_if(unodeDigraphs.begin(),unodeDigraphs.end(),GraphNumber(u->id()))
              == unodeDigraphs.end())
      {
        gU = new Graph();
        // number of digraphs
        gU->index(u->id());
        // add nodes
        gU->makeSubgraph(nodesBeg,nodesEnd,false);
      }

      if (find_if(unodeDigraphs.begin(),unodeDigraphs.end(),GraphNumber(v->id()))
              == unodeDigraphs.end())
      {
        gV = new Graph();
        // number of digraphs
        gV->index(v->id());
        // add nodes
        gV->makeSubgraph(nodesBeg,nodesEnd,false);
      }
      
      // initialize the map Node <-> (Path, marked, distance)
      // and the nodeSet needed for dijkstra
      initNodeMap(*u, comp, nodeMapU, nodeSetU);
      initNodeMap(*v, comp, nodeMapV, nodeSetV);

      // shortest path from u to all other nodes
      dijkstra(*u,nodeMapU,nodeSetU,false);
      dijkstra(*v,nodeMapV,nodeSetV,false);

      UPATH("**************************** new pair: "+Ttos(*u))
      UPATH("-"+Ttos(*v)+"\n")
      #if GRAPHLIB_DEBUG==6
        writeNodeMap(nodeMapU,upath.os());
        writeNodeMap(nodeMapV,upath.os());
      #endif

      // forall nodes in this component
      for (w=nodesBeg; w!=nodesEnd; w++)
      {
        // w must not be u or v
        if (*w == *u || *w == *v)
          continue;

        // position in the maps
        wU = nodeMapU.find(*w);
        wV = nodeMapV.find(*w);

        // if w adjacent to u -> into digraph
        if (gU && wU->second.distance==1)
          try
          {
            gU->newEdge(w->id(),u->id());
          }
          catch (DoubleError&) {;}

        // if w adjacent to v -> into digraph
        if (gV && wV->second.distance==1)
          try
          {
            gV->newEdge(w->id(),v->id());
          }
          catch (DoubleError&) {;}

        //
        //// calculate even prototypes
        //
        // if the distance to both U-Nodes equals
        if ((distance=wU->second.distance) == wV->second.distance)
        {

          // adjacent node to w = x and y
          for (x_edge=w->adjBegin(); x_edge!=w->adjEnd(); x_edge++)
          {
            y_edge=x_edge;
            for (++y_edge; y_edge!=w->adjEnd(); y_edge++)
            {
              // x
              xU = nodeMapU.find(w->opposite(*x_edge));
              xV = nodeMapV.find(w->opposite(*x_edge));
              // y
              yU = nodeMapU.find(w->opposite(*y_edge));
              yV = nodeMapV.find(w->opposite(*y_edge));

              UPATH("\nw: "+Ttos(*w)+"\tx: "+Ttos(xU->first.id()))
              UPATH("\ty: "+Ttos(yU->first.id()))

              // if x belongs to a shortest path from u to w -> into digraph
              if (gU && xU->second.distance+1 == distance)
                try
                {
                  gU->newEdge(w->id(),xU->first.id());
                }
                catch (DoubleError&) {;}

              // if x belongs to a shortest path from v to w -> into digraph
              if (gV && xV->second.distance+1 == distance)
                try
                {
                  gU->newEdge(w->id(),xU->first.id());
                }
                catch (DoubleError&) {;}

              // if y belongs to a shortest path from u to w -> into digraph
              if (gU && yU->second.distance+1 == distance)
                try
                {
                  gU->newEdge(w->id(),yU->first.id());
                }
                catch (DoubleError&) {;}

              // if y belongs to a shortest path from v to w -> into digraph
              if (gV && yV->second.distance+1 == distance)
                try
                {
                  gU->newEdge(w->id(),yU->first.id());
                }
                catch (DoubleError&) {;}

              // check if u-x-w and v-y-w == short
              if (xU->second.distance+1 == distance &&
                  yV->second.distance+1 == distance)
              {
                // new Prototype
                b = new Prototype(edges.size());
                b->setNodes(w->id(),u->id(),v->id(),xU->first.id(),yV->first.id());
                // P(u,x)
                b->addNodes(xU->second.path.begin(),xU->second.path.end());
                // {w}
                b->addNodes(*w);
                // P(y,v)
                b->addNodes(yV->second.path.rbegin(),yV->second.path.rend());
                // calcCycle == false, if one occures edge twice
                if (b->calcCycle(edgeMap) == true)
                {
                  b->setmembers(Cycle::UPath);
                  // add to prototypes
                  prototypes.push_back(*b);
                  UPATH("\neven:\t"+Ttos(*b)+"\n")
                }
                delete b;
              }

              // or if u-y-w and v-x-w == short
              if (yU->second.distance+1 == distance &&
                  xV->second.distance+1 == distance)
              {
                b = new Prototype(edges.size());
                b->setNodes(w->id(),u->id(),v->id(),yU->first.id(),xV->first.id());
                // P(u,y)
                b->addNodes(yU->second.path.begin(),yU->second.path.end());
                // {w}
                b->addNodes(*w);
                // P(x,v)
                b->addNodes(xV->second.path.rbegin(),xV->second.path.rend());
                // calcCycle == false, if one occures edge twice
                if (b->calcCycle(edgeMap) == true)
                {
                  b->setmembers(Cycle::UPath);
                  // add to prototypes
                  prototypes.push_back(*b);
                  UPATH("\neven_yx:\t"+Ttos(*b)+"\n")
                }
                delete b;
              }
            }
          }
        }

        //
        //// calculate even prototypes
        //
        // if the distance to both U-nodes is not the same
        else
        {
          // x = w, adjacent node = y
          for (y_edge=w->adjBegin(); y_edge!=w->adjEnd(); y_edge++)
          {
            // y
            yU = nodeMapU.find(w->opposite(*y_edge));
            yV = nodeMapV.find(w->opposite(*y_edge));

            UPATH("\nx: "+Ttos(*w))
            UPATH("\ty: "+Ttos(yU->first.id()))

            // if y between w(x) and u -> into digraph
            if (gU && wU->second.distance+1 == yU->second.distance)
              try
              {
                gU->newEdge(*w,yU->first);
              }
              catch (DoubleError&) {;}

            // if y between w(x) and v -> into digraph
            if (gV && wV->second.distance+1 == yV->second.distance)
              try
              {
                gV->newEdge(*w,yV->first);
              }
              catch (DoubleError&) {;}

            // if u-x(w)-y-v == short
            // P = (P(ux)+e(xy)+P(yv))   ... distance1
            // P(ux) < 1/2 P  &&  P(vy) < 1/2 P
            unsigned distance1 = wU->second.distance+1+yV->second.distance;
            unsigned distance2 = yU->second.distance+1+wV->second.distance;
            if (2*wU->second.distance < distance1 &&
                2*yV->second.distance < distance1)
            {
              b = new Prototype(edges.size());
              b->setNodes(INFINITY,u->id(),v->id(),w->id(),yV->first.id());
              // P(u,x)
              b->addNodes(wU->second.path.begin(),wU->second.path.end());
              // P(y,v)
              b->addNodes(yV->second.path.rbegin(),yV->second.path.rend());
              // calcCycle == false, if one occures edge twice
              if (b->calcCycle(edgeMap) == true)
              {
                b->setmembers(Cycle::UPath);
                // add to prototypes
                prototypes.push_back(*b);
                UPATH("\nodd:\t"+Ttos(*b)+"\n")
              }
              delete b;
            }

            // if u-y-x(w)-v == short
            // P = (P(uy)+e(yx)+P(xv))   ... distance2
            // P(uy) < 1/2 P  &&  P(yx) < 1/2 P
            // P(
            else if (2*yU->second.distance < distance2 &&
                     2*wV->second.distance < distance2)
            {
              b = new Prototype(edges.size());
              b->setNodes(INFINITY,u->id(),v->id(),yU->first.id(),w->id());
              // P(u,y)
              b->addNodes(yU->second.path.begin(),yU->second.path.end());
              // P(y,v)
              b->addNodes(wV->second.path.rbegin(),wV->second.path.rend());
              // calcCycle == false, if one occures edge twice
              if (b->calcCycle(edgeMap) == true)
              {
                b->setmembers(Cycle::UPath);
                // add to prototypes
                prototypes.push_back(*b);
                UPATH("\nodd_yx:\t"+Ttos(*b)+"\n")
              }
              delete b;
            }
          }
        }
      }

      // if a digraph exists -> put into list
      if (gU)
      {
        unodeDigraphs.push_back(gU);
        gU = 0;
      }
      if (gV)
      {
        unodeDigraphs.push_back(gV);
        gV = 0;
      }
    }
  }

  // cycles in order of their weight (lenght)
  prototypes.sort(lessLength());
  UPATH("\n\nUPATH\n"+Ttos(prototypes)+"\nENDE\n")
}

void CycleGraph::calcCI(const Component& biComp,
                        list<Prototype>& prototypes)
{
  #if GRAPHLIB_DEBUG==6
    static bool ersteMal=true;
    LogFile algo;
    if (ersteMal)
    {
      algo.init("vismara_algo1.graphlib");
      ersteMal=false;
    }
    else
      algo.reopen("vismara_algo1.graphlib");
    #define ALGO(msg) algo.logging(msg);
  #else
    #define ALGO(msg) ;
  #endif

  // (same names as in the vismara paper)
  constNodeIter r, y, z, p, q;
  NodePathMap::iterator yI, zI, pI, qI;
  list<Node> s, vr;
  constEdgeIter z_edge;

  // for dijkstra
  NodePathMap nodeMap;
  NodeSet     nodeSet;

  Prototype *b;
  Graph *d;

  // for all nodes of the biconnected component
  for(r=biComp.nodesBegin(); r!=biComp.nodesEnd(); r++)
  {
    // initialize the map Node <-> (Path, marked, distance)
    // and the nodeSet needed for dijkstra
    initNodeMap(*r, biComp, nodeMap, nodeSet);

    // shortest path from r to all other nodes - order important
    dijkstra(*r,nodeMap,nodeSet,true);

    ALGO("**************************** new node: "+Ttos(*r)+"\n")
    #if GRAPHLIB_DEBUG==6
      writeNodeMap(nodeMap,algo.os());
    #endif

    // calculating Vr (nodes with shortest Path only preceeding r
    // and set diGraph(Vr)
    for(constNodeIter i=biComp.nodesBegin(); i!=r; i++)
      if (nodeMap[*i].path.size() != 0)
        vr.push_back(*i);

    vr.push_back(*r);
    ALGO("Vr: "+Ttos(vr)+"\n")

    // digraph for calculating relevant cycles
    d=new Graph();
    // number of digraph = r.id
    d->index(r->id());
    // all nodes form Vr are contained in d, edges added later
    d->makeSubgraph(vr.begin(),vr.end(),false);

    // remove r from Vr
    vr.pop_back();

    // for all nodes from Vr
    for(y=vr.begin(); y!=vr.end(); y++)
    {
      ALGO("y: "+Ttos(y->id()))
      yI = nodeMap.find(*y);

      // S = {}
      s.clear();

      // set Ur = Ur + {y,r}
      if (yI->second.distance == 1)
        d->newEdge(y->id(),r->id());

      // all neighbours to y
      for (z_edge=y->adjBegin(); z_edge!=y->adjEnd(); z_edge++)
      {
        ALGO("\nz_edge: "+Ttos(*z_edge))
        // found neighbour z is in Vr
        if ((z=std::find(vr.begin(),vr.end(),y->opposite(*z_edge))) != vr.end())
        {
          ALGO("\n\tz= "+Ttos(z->id()))
          zI = nodeMap.find(*z);

          // z belongs to a shortest path from r to y
          if (zI->second.distance + 1 == yI->second.distance)
          {
            // S = S + {z}
            s.push_back(*z);

            // set Ur = Ur + {y,z}
            d->newEdge(y->id(),z->id());
          }

          // z < y; d(r,z) != d(r,y) + 1; P(r,y) and P(r,z} == {r} -> odd cycle
          else if ((z->id()<y->id()) &&
                    zI->second.distance!=yI->second.distance+1 &&
                    yI->second.intersectionPath(nodeMap[*z],*r))
          {
            // new Prototype
            b = new Prototype(edges.size());
            b->setNodes(r->id(),y->id(),z->id());
            // P(r,z)
            b->addNodes(zI->second.path.begin(),zI->second.path.end());
            // P(y,r)
            b->addNodes(yI->second.path.rbegin(),yI->second.path.rend());
            // calc cycle P(r,z) + (r,z) + P(y,r)
            b->calcCycle(edgeMap);
            // add to prototypes
            prototypes.push_back(*b);
            ALGO("\nodd:  "+Ttos(*b)+"\n")
            delete b;
          }
        }
      }

      ALGO("\nS: "+Ttos(s)+"\n")
      // forall p in S
      for(p=s.begin(); p!=s.end(); p++)
      {
        pI = nodeMap.find(*p);
        q=p;
        q++;
        // for all q in S
        for(;q!=s.end();q++)
        {
          qI = nodeMap.find(*q);
          // intersection of P(r,p) and P(r,q) == {r} -> even cycle
          if (pI->second.intersectionPath(qI->second,*r))
          {
            ALGO(Ttos(p->id())+" + "+Ttos(q->id())+" -> ")
            // new Prototype
            b = new Prototype(edges.size());
            b->setNodes(r->id(),p->id(),q->id(),y->id());
            // P(r,p)
            b->addNodes(pI->second.path.begin(),pI->second.path.end());
            // {y}
            b->addNodes(*y);
            // P(q,r)
            b->addNodes(qI->second.path.rbegin(),qI->second.path.rend());
            // calc cycle P(r,p) + {p,y} + {y,q} + P(q,r)
            b->calcCycle(edgeMap);
            // add to prototypes
            prototypes.push_back(*b);
            ALGO("\neven: "+Ttos(*b)+"\n")
            delete b;
          }
        }
      }
    }

    digraphs.push_back(d);
    vr.erase(vr.begin(), vr.end());
  }

  // cycles in order of their weight (lenght)
  prototypes.sort(lessLength());
  ALGO("\n\nALGROITHM 1\n"+Ttos(prototypes)+"\nENDE\n")
}

void CycleGraph::calcPrototypes(list<Prototype>& prototypes,
                                unsigned long& protoIndex,
                                int cyclom, DecompositionsMode flag)
{
  // nothing to do
  if (prototypes.size() == 0)
    return;

  #if GRAPHLIB_DEBUG==6
    static bool ersteMal=true;
    LogFile proto;
    if (ersteMal)
    {
      proto.init("proto.graphlib");
      ersteMal=false;
    }
    else
      proto.reopen("proto.graphlib");
    #define PROTO(msg) proto.logging(msg);
  #else
    #define PROTO(msg) ;
  #endif

  list<Prototype>::iterator it = prototypes.begin();
  list<Prototype>::iterator  cycle = it;
  list<BitVector> gaussLess, gaussLEq, less;

  int length=0, rank=0, rankg=0;
  bool ende=false;

  // clear all lists;
  gaussLess.clear();
  gaussLEq.clear();
  less.clear();

  while (it != prototypes.end())
  {
    cycle = it++;
    PROTO("\n##### cyclomatic number "+Ttos(cyclom)+"\ni= "+Ttos(*cycle)+"\n")

    // new length
    if (length != cycle->count())
    {
      // still missing cycles in mcb
      // old gaussLEq == less (all cycles less or equal length)
      if (cyclom)
      {
        PROTO("** new length less\n")
        // new length
        length = cycle->count();
        // less = gaussLEq
        less = gaussLEq;
      }

      // all cycles for mcb found -> end
      else
      {
        // end after this length
        ende = true;
        prototypes.erase(cycle, prototypes.end());
        it = prototypes.end();
        PROTO("** ENDE  ***\n")
      }
    }

    // not stopped
    if (!ende)
    {
      // gaussLess = Gauss(all less cyles) + {C}
      gaussLess = less;
      gaussLess.push_back(*cycle);

      rank = gaussLess.size();

      PROTO("** less = less + i ?\n"+Ttos(less)+"\n"+Ttos(gaussLess))

      // B< + {C} lin.independent ?
      if (rank==1 || rank == (rankg=gauss(gaussLess, gaussLess.begin(), 0)))
      {
        PROTO("after gauss1:\n"+Ttos(gaussLess))
        // current cycle == Prototype
        cycle->setmembers(Cycle::Prototype);
        cycle->prototypeIndex(protoIndex++);

        //( gaussLEq = gauss(less+equals)
        gaussLEq.push_back(*cycle);
        rank = gaussLEq.size();

        PROTO("********* less/equ + i ?\n"+Ttos(gaussLEq))

        // B< + {C} + B= lin.independent ?
        if (rank == (rankg=gauss(gaussLEq, gaussLEq.begin(), 0)))
        {
          PROTO("after gauss2:\n"+Ttos(gaussLEq))
          // current cycle == member of mcb -> reduce cyclomatic number
          // and set flag
          cycle->setmembers(Cycle::Mcb);
          cyclom--;
          if (flag == Horton)
            cycles.push_back(new Cycle(*cycle));

          PROTO("=== MCB\n"+Ttos(*cycle))
        }

        if (flag == Prototypes)
          cycles.push_back(new Cycle(*cycle));
        PROTO("=== PROTO\n"+Ttos(*cycle))
      }

      // no prototype -> remove from list
      else
        prototypes.erase(cycle);

    }
  }

  PROTO("\n\nPROTOTYPES\n"+Ttos(prototypes)+"\nENDE\n")
}

void CycleGraph::calcRelevants(list<Prototype>& prototypes,
                               list<Cycle*>& relevants, list<Cycle*>&mcb)
{
  // nothing to do
  if (prototypes.size() == 0)
    return;

  #if GRAPHLIB_DEBUG==6
    static bool ersteMal=true;
    LogFile relev;
    if (ersteMal)
    {
      relev.init("relev.graphlib");
      ersteMal=false;
    }
    else
      relev.reopen("relev.graphlib");
    #define RELEV(msg) relev.logging(msg);
  #else
    #define RELEV(msg) ;
  #endif

  // number of the prototype familiy
  unsigned long prototypeIndex;
  list<Prototype>::iterator cycle = prototypes.begin();
  Path currentPath;
  // Pathes p->r, q->r
  vector<Path> resultP, resultQ;
  vector<Path>::iterator prI, qrI;
  // digraph
  list<Graph*>::iterator dI;

  // for cycle - calculations
  list<Node> nodeList;
  vector<int>nodes;
  Cycle *b;

  prototypes.sort(lessLength());

  // triangles and rectangels -> no other members in the prototype family
  while (cycle != prototypes.end() && cycle->count()<5)
  {
    // new cycle -> add to list of all relevant cycles
    b = new Cycle(*cycle++);
    cycles.push_back(b);
    b->setmembers(Cycle::Relevant);
    relevants.push_back(b);
    RELEV(Ttos(b));
    // if it's a member of the mcb -> put into mcb
    if (b->getmembers(Cycle::Mcb))
      mcb.push_back(b);
  }

  // all others -> calculates relevant cycles form the prototype
  for ( ; cycle!=prototypes.end(); cycle++)
  {
    // if a u-path -> next one
    if (cycle->getmembers(Cycle::UPath))
    {
      // new path -> add to list of all relevant cycles
      b = new Cycle(*cycle);
      cycles.push_back(b);
      b->setmembers(Cycle::Relevant);
      relevants.push_back(b);
      RELEV(Ttos(b));
      // if it's a member of the mcb -> put into mcb
      if (b->getmembers(Cycle::Mcb))
        mcb.push_back(b);
      continue;
    }

    // number of the prototype of this family
    prototypeIndex = cycle->prototypeIndex();

    // find diGraph with number == r
    dI = find_if(digraphs.begin(),digraphs.end(),GraphNumber(cycle->r()));
    if (dI == digraphs.end())
      THROW(logic_error,"CycleGraph::calcRelevants(list<Prototype>&,"
            "list<Cycle*>&,list<Cycle*>&) can not find digraph"
            +Ttos(cycle->r()))

    // all pathes P(p,r)
    resultP = (*dI)->vectorPaths(cycle->p(),cycle->r(),currentPath);
    // all pahtes P(q,r)
    resultQ = (*dI)->vectorPaths(cycle->q(),cycle->r(),currentPath);

    //even cycles (gerade)
    // if (cycle->length() % 2 == 0)
    if (cycle->x() != INFINITY)
    {
      // forall P(p,r)
      for (prI=resultP.begin(); prI!=resultP.end(); prI++)
      {
        // forall P(q,r)
        for (qrI=resultQ.begin(); qrI!=resultQ.end(); qrI++)
        {
          nodeList.clear();
          // P(r,p)
          int2Nodes(nodeList,*prI,false);
          // {x}
          int2Nodes(nodeList,cycle->x());
          // P(q,r)
          int2Nodes(nodeList,*qrI,true);
          // new Cycle
          b = new Cycle(edges.size());
          // nodeList -> cycle
          b->addNodes(nodeList.begin(),nodeList.end());
          RELEV("prI "+Ttos(*prI)+"\nqrI "+Ttos(*qrI)+"\nx ")
          RELEV(Ttos(cycle->x())+"\n")
          // calc cycle P(r,p) + {p,x} + {x,q} + P(q,r)
          b->calcCycle(edgeMap);
          // set number of prototype
          b->prototypeIndex(prototypeIndex);
          // add cycle to list of all cycles of th graph
          cycles.push_back(b);
          b->setmembers(Cycle::Relevant);
          //              list of all cycles of the 2-conn comp
          relevants.push_back(b);
          // if the cycle is the prototype of this family ...
          if ((Cycle)(*cycle) == *b)
          {
            // set flag prototype
            b->setmembers(Cycle::Prototype);
            // and if it is in the mcb
            if (cycle->getmembers(Cycle::Mcb))
            {
              // set flag mcb
              b->setmembers(Cycle::Mcb);
              // and put it into mcb
              mcb.push_back(b);
            }
          }
          RELEV(Ttos(b))
        }
      }
    }

    // odd cycles (ungerade)
    else
    {
      // forall P(p,r)
      for (prI=resultP.begin(); prI!=resultP.end(); prI++)
      {
        // forall P(q,r)
        for (qrI=resultQ.begin(); qrI!=resultQ.end(); qrI++)
        {
          nodeList.clear();
          // P(r,p)
          int2Nodes(nodeList,*prI,false);
          // P(q,r)
          int2Nodes(nodeList,*qrI,true);
          // new Cycle
          b = new Cycle(edges.size());
          // nodeList -> cycle
          b->addNodes(nodeList.begin(),nodeList.end());
          RELEV("prI "+Ttos(*prI)+"\nqrI "+Ttos(*qrI)+"\n")
          // calc cycle P(r,p) + {p,q} + P(q,r)
          b->calcCycle(edgeMap);
          // set number of prototype
          b->prototypeIndex(prototypeIndex);
          // add cycle to list of all cycles of th graph
          cycles.push_back(b);
          b->setmembers(Cycle::Relevant);
          //              list of all cycles of the 2-conn comp
          relevants.push_back(b);
          // if the cycle is the prototype of this family ...
          if ((Cycle)(*cycle) == *b)
          {
            // set flag prototype
            b->setmembers(Cycle::Prototype);
            // and if it is in the mcb
            if (cycle->getmembers(Cycle::Mcb))
            {
              // set flag mcb
              b->setmembers(Cycle::Mcb);
              // and put it into mcb
              mcb.push_back(b);
            }
          }
          RELEV(Ttos(b))
        }
      }
    }
  }

  // delete digraphs
  deleteDiGraphs();
}

void CycleGraph::calcXchgClasses(list<Cycle*>& relevants,unsigned long& xchgIndex)
{
  // nothing to do
  if (relevants.size() == 0)
    return;

  #if GRAPHLIB_DEBUG>4
    static bool ersteMal=true;
    LogFile xchg;
    if (ersteMal)
    {
      xchg.init("xchg.graphlib");
      ersteMal=false;
    }
    else
      xchg.reopen("xchg.graphlib");
    #define XCHG(msg) xchg.logging(msg);
  #else
    #define XCHG(msg) ;
  #endif

  XCHG("relevant cycles:\n"+Ttos(relevants))

  CycleIter cycle = relevants.begin();

  // only one cycle -> own xchg-class == essential
  if (relevants.size() == 1)
  {
    XCHG("one relevant: "+Ttos(xchgIndex)+"\n"+Ttos(*cycle))
    (*cycle)->xchgClassIndex(xchgIndex++);
    (*cycle)->setmembers(Cycle::Essential);
    XCHG("only one relevant -> one xchg-class\n ***** XCHG ENDE ******\n")
    return;
  }

  int length = (*cycle)->count();

   // relevant cycles of same length
  vector<Cycle*> equals;

  // cycles of an MCB with less and equal length
  list<Cycle*> lessM;
  list<Cycle*> equalM;

  // matrix of cycles
  list<BitVector>* cycleMatrix = NULL;

  // BitVector to demask the "1" from the cycles of M<
  BitVector* mask;

  /*      for all relevant cycles of this component */
  while (cycle != relevants.end())
  {
    // cycle is in an MCB -> put into equalM
    if ((*cycle)->getmembers(Cycle::Mcb))
      equalM.push_back(*cycle);

//    cout << *cycle;

    // all cycles of same length
    equals.push_back(*cycle++);

    // end of cycles or new length -> calc essential in equals
    if ((cycle==relevants.end()) || ((*cycle)->count()>length))
    {
//      cout << "lessM\n" << lessM << endl;
//      cout << "equalM\n" << equalM << endl;
//      cout << "equals\n " << equals << endl;

      // if only one relevant cycle of this length -> own class == essential
      if (equals.size() == 1)
      {
        XCHG("one in length: "+Ttos(xchgIndex)+"\n"+Ttos(*equals.begin()))
        (*equals.begin())->xchgClassIndex(xchgIndex++);
        (*equals.begin())->setmembers(Cycle::Essential);
      }

      else
      {
        // initialize matrix of cycles: A <- (M<, R=)
        cycleMatrix = makeCycleMatrix(lessM,equals);

        // for demask MCB-Cycles: "1" for all cycles of equals
        mask = new BitVector(lessM.size()+equals.size());
        mask->setBit(lessM.size(),lessM.size()+equals.size());

        // Gauss-Jordan elimination
        gaussJordan(*cycleMatrix,cycleMatrix->begin(),0);
        XCHG("gauss Jordan:\n"+Ttos(*cycleMatrix))

        // Color Matrix and make xchg-classes (assign xchgIndex)
        colorCycleMatrix(*cycleMatrix,equals,xchgIndex,*mask);

        if (cycleMatrix)
          delete cycleMatrix;
      }

      // not at the end -> set new length
      if (cycle != relevants.end())
        length = (*cycle)->count();
      else
        length = 0;

      // M< += M=
      lessM.insert(lessM.end(),equalM.begin(),equalM.end());
      // M= = R= = \emtpyset
      equalM.clear();
      equals.clear();
    }
  }
  XCHG("\n\n ************* XCHG ENDE *************\n\n")
}

list<BitVector>* CycleGraph::makeCycleMatrix(const list<Cycle*> lessM,
                                             const vector<Cycle*> equals) const
{
  // a new matrix of cycles (column = cycle)
  list<BitVector>* matrix = new list<BitVector>();

  // a row of the matrix (= edges)
  BitVector row(lessM.size()+equals.size());

  unsigned posx, posy;
  constCycleIter it;
  constCycleIter end = lessM.end();
  vector<Cycle*>::const_iterator relev;
  vector<Cycle*>::const_iterator relEnd = equals.end();

  // fill CycleMatrix
  // edges of the cycles
  for (posx=0; posx!=edges.size(); posx++)
  {
    // new row
    posy=0;

    // cycle of lessM
    it = lessM.begin();
    while (it != end)
    {
      // if this edge in the cycle -> "1" in the CycleMatrix
      if ((*it)->getBit(posx))
        row.setBit(posy);

      it++; posy++;
    }

    // cycle of equals
    relev = equals.begin();
    while (relev != relEnd)
    {
      // if this edge in the cycle -> "1" in the CycleMatrix
      if ((*relev)->getBit(posx))
        row.setBit(posy);

      relev++; posy++;
    }

    // count "1" in the row
    row.setLength();

    // if this edge is contained in any cycle (of this block)
    //  -> add row to cycleMatrix and clear for next edge
    if (!row.empty())
    {
      matrix->push_back(row);
      row.clearAllBits();
    }
  }

  return matrix;
}

void CycleGraph::colorCycleMatrix(const list<BitVector>& matrix,
                                  vector<Cycle*>& equals,
                                  unsigned long& xchgIndex,
                                  const BitVector& mask) const
{
  #if GRAPHLIB_DEBUG>4
    static bool ersteMal=true;
    LogFile xchg;
    if (ersteMal)
    {
      xchg.init("colorxchg.graphlib");
      ersteMal=false;
    }
    else
      xchg.reopen("colorxchg.graphlib");
    #define XCHG(msg) xchg.logging(msg);
  #else
    #define XCHG(msg) ;
  #endif

  XCHG("\nCOLORING\n")

  vector<Cycle*>::iterator cycle = equals.begin();

  // if cycleMatrix == diagonal matrix -> all cycle essential
  if (matrix.size() == matrix.begin()->getsize())
  {
    while (cycle != equals.end())
    {
      XCHG("\ndiagonal: "+Ttos(xchgIndex)+"\t"+Ttos(*cycle))
      (*cycle)->setmembers(Cycle::Essential);
      (*cycle++)->xchgClassIndex(xchgIndex++);
    }
    return;
  }

  /*   C O L O R I N G   */
  // actual row and end of the matrix
  list<BitVector>::const_iterator row = matrix.begin();
  list<BitVector>::const_iterator end = matrix.end();

  // for identification of colors
  BitVector help(*row++);

  // list of different colors
  list<BitVector>* colors = new list<BitVector>();
  list<BitVector>::iterator thisColor, foundColor;

  //  put the first row
  colors->push_back(help);

  bool newColor;

  // for all rows of the cycle matrix
  for (;row != end; row++)
  {
    // initialize
    newColor=true, foundColor=colors->end();

    // for all colors
    for (thisColor=colors->begin(); thisColor!=colors->end(); thisColor++)
    {
      // any A_{ij'} = A_{ij''} = 1 ?
      help = *thisColor & *row & mask;

      // row has the same color as thisColor
      if (!help.empty())
      {
        // if already belongs to another color -> identify them
        if (newColor == false)
        {
          // identify the two colors
          *thisColor |= *foundColor;

          // erase thisColor
          colors->erase(foundColor);

          // thisColor is the new foundColor
          foundColor = thisColor;
        }

        // first color for this row
        else
        {
          // no new color
          newColor = false;

          // add this row to this color
          *thisColor |= (*row & mask);

          // found a color (needed if the row would belong to two colors)
          foundColor = thisColor;
        }
      }
    }

    // row belongs to none of the existing color classes
    if (newColor)
      colors->push_back(*row);
  }

//  cout << "\nColors\n" << *colors << endl;

  /*       C O L O R S =>  <-> classes   */
  thisColor = colors->begin();
  cycle = equals.begin();
  int firstRelevant = thisColor->getsize() - equals.size();
  int size = thisColor->getsize();
  bool essential;

  // all colors
  for ( ;thisColor!=colors->end(); thisColor++,xchgIndex++)
  {
    XCHG("\ncolor: "+Ttos(xchgIndex)+"\n")
    if (thisColor->count() == 1)
      essential=true;
    else
      essential=false;

    // all cycles
    for (int i=firstRelevant; i!=size; i++)
      // cycle on position i belongs to this class
      if (thisColor->getBit(i))
      {
        if (essential)
          equals[i-firstRelevant]->setmembers(Cycle::Essential);

        XCHG(Ttos(equals[i-firstRelevant]))
        equals[i-firstRelevant]->xchgClassIndex(xchgIndex);
      }
   }
}

void CycleGraph::calcShortests(list<Cycle*>& relevants)
{
  // nothing to do
  if (relevants.size() == 0)
    return;

  #if GRAPHLIB_DEBUG==6
    static bool ersteMal=true;
    LogFile edge_short;
    if (ersteMal)
    {
      edge_short.init("short.graphlib");
      ersteMal=false;
    }
    else
      edge_short.reopen("short.graphlib");
    #define SHORT(msg) edge_short.logging(msg);
  #else
    #define SHORT(msg) ;
  #endif

  Cycle* unique=0;
  CycleIter cycle;
  unsigned edge;
  int length;

  // forall edges
  for (edge=0; edge!=edges.size(); edge++)
  {
    SHORT("\nedge_nr: "+Ttos(edge))
    // forall cycles
    length = 0;
    unique = 0;
    for (cycle=relevants.begin(); cycle!=relevants.end(); cycle++)
    {
      // if cycle contains this edge ...
      if ((*cycle)->getBit(edge))
      {
        // ... it is edge-short,
        //  if it is the first cycle for this edge
        if (length == 0)
        {
          SHORT(" short: "+Ttos(*cycle))
          // set length of short cycles
          length = (*cycle)->count();
          // set flag - edge-short
          (*cycle)->setmembers(Cycle::Shortest);
          // found one -> maybe unique
          unique = *cycle;
        }
        // else if the short cycles found jet, have the same length
        else if (length == (*cycle)->count())
        {
          SHORT(" short: "+Ttos(*cycle))
          // set flag - edge-short
          (*cycle)->setmembers(Cycle::Shortest);
          // not unique now
          unique = 0;
        }
        // if cycle is longer than short ones -> END for this edge
        else
          cycle = --relevants.end();
      }
    }

    // if there is a unique cycle -> set flag
    if (unique)
    {
      SHORT("\t\tunique: "+Ttos(unique))
      unique->setmembers(Cycle::UniqueShortest);
    }
  }
  SHORT("\n\n ********* EDGE-SHORT: ENDE ********\n\n")
}

void CycleGraph::calcEssentials(list<Cycle*>& relevants, list<Cycle*>& mcb)
{
  // nothing to do
  if (relevants.size() == 0)
    return;

  #if GRAPHLIB_DEBUG==6
    static bool ersteMal=true;
    LogFile essential;
    if (ersteMal)
    {
      essential.init("essential.graphlib");
      ersteMal=false;
    }
    else
      essential.reopen("essential.graphlib");
    #define ESS(msg) essential.logging(msg);
  #else
    #define ESS(msg) ;
  #endif

  CycleIter cycle = relevants.begin();
  CycleIter it, i;

  if (relevants.size() == 1)
  {
    (*cycle)->setmembers(Cycle::Essential);
    ESS("only one relevant -> essential\n ***** ESSENTIAL ENDE ******\n")
    return;
  }

  int length = (*cycle)->count();
  int cyclom = mcb.size();

  // cycles of the same length
  list<Cycle*> equals;
  equals.clear();

  // for gaussian elimination - matrix for independent check
  list<BitVector> help;


  // forall relevant cycles
  while (cycle != relevants.end())
  {
    // all cycles of same length
    equals.push_back(*cycle++);

    // end of cycles or new length -> calc essential in equals
    if ((cycle==relevants.end()) || ((*cycle)->count()>length))
    {
      // for all relevants
      for (it=relevants.begin(); it!=relevants.end(); it++)
      {
        // is in mcb and same length as equals
        if ((*it)->getmembers(Cycle::Mcb) && (*it)->count()==length)
        {
          help.clear();
          // help = equals
          help.insert(help.end(),equals.begin(),equals.end());
//          cout << "equals: \n" << help << endl;
          // help = equals \ {C}
          help.erase(find(help.begin(),help.end(),**it));
//          cout << "equals - {C}\n" << help << endl;
          // help = equals + mcb \ {C} ( add mcb \ {C})
          for (i=mcb.begin(); i!=mcb.end(); i++)
          {
//            cout << "mcb: " << *i;
            // if not {C} and not already in help -> add
            if (*i!=*it && find(help.begin(),help.end(),*i)==help.end())
              help.push_back(*i);
          }

//          cout << "equals + mcb - {C}\n" << help << endl;
          // is equals + mcb \ {C} lin. independent ?
          if (gauss(help,help.begin(),0) < cyclom)
            // set flag - essential
            if ((*it)->getmembers(Cycle::Essential) == false)
              cout << "FEHLER";
       }
     }

     // set length
     if (cycle != relevants.end())
      length = (*cycle)->count();

      // empty equals
      equals.clear();
    }
  }
  ESS("\n\n ************* ESSENTIAL ENDE *************\n\n")
}

void CycleGraph::int2Nodes(list<Node>& liste, int it) const
{
  constNodeIter node;
  if ((node=findNode(it)) == nodes.end())
    THROW(runtime_error,"CycleGraph::int2Nodes(list<Node>&,int)"
          " const: unvalide node"+Ttos(it))

  liste.push_back(*node);
}

void CycleGraph::int2Nodes(list<Node>& liste, const Path& path,
                                bool right) const
{
  // right orientation
  if (right)
  {
    Path::const_iterator it  = path.begin();
    Path::const_iterator end = path.end();
    while (it != end)
      int2Nodes(liste,*it++);
  }
  else
  {
    Path::const_reverse_iterator it  = path.rbegin();
    Path::const_reverse_iterator end = path.rend();
    while (it != end)
      int2Nodes(liste,*it++);
  }
}

void CycleGraph::init()
{
  // pairs each edge with an unique id
  initEdgeIdMap();
  LOG1("Edge <=> id\n")
  LOG1(Ttos(edgeMap))

  // calculated biconnected components, if they have not been calculcated yet
  if (biComponents.empty())
    calcComponents();

  // separate U-nodes to the components
  unodes2Comp();
}

void CycleGraph::unodes2Comp()
{
  cVNodeIter it = unodes.begin();
  cVNodeIter end= unodes.end();

  constSubGraphIter comp = conComponents.begin();
  constSubGraphIter comE = conComponents.end();

  list<Node> uComp;

  // for all connected component
  while (comp != comE)
  {
    constNodeIter nodesBeg = (*comp)->nodesBegin();
    constNodeIter nodesEnd = (*comp)->nodesEnd();
    constNodeIter unode;
    uComp.clear();

    // for all unodes
    for (it=unodes.begin();it != end;it++)
    {
      unode=find(nodesBeg,nodesEnd,*it);
      if (unode != nodesEnd)
        uComp.push_back(*unode);
    }
    unodesComp.push_back(uComp);
    comp++;
  }
}

void CycleGraph::initEdgeIdMap()
{
  constEdgeIter it  = edges.begin();
  constEdgeIter end = edges.end();
  unsigned i = 0;

  while (it != end)
  {
    edgeMap.insert(edgeMap.end(),make_pair(*it++,i++));
  }
}

void CycleGraph::setCycleIndex(bool isUBasis)
{
  // sort all cycles by length
  cycles.sort(lessCycleLength());

  CycleIter it  = cycles.begin();
  CycleIter end = cycles.end();
  unsigned long i = 1;

  // if cycle basis -> into cycleVec
  if (! isUBasis)
    while (it != end)
    {
      cycleVec.push_back(*it);
      (*it++)->index(i++);
    }
  // if U basis -> into uElemVec
  else
    while (it != end)
    {
      uElemVec.push_back(*it);
      (*it++)->index(i++);
    }
      
  // cycles stored in vectors now
  cycles.clear();
}

void CycleGraph::copyCyclePrototypes2UElem(list<Prototype>& prototypes)
{
  CycleIter it  = cycles.begin();
  CycleIter end = cycles.end();

  // for all cycles
  while (it != end)
  {
    if ((*it)->getmembers(Cycle::Prototype))
      prototypes.push_back(**it);

    it++;
  }

}

void CycleGraph::copyCycleRelevants2UElem()
{


}
void CycleGraph::writeNodeMap(const NodePathMap& nodeMap, ostream& os) const
{
  NodePathMap::const_iterator it  = nodeMap.begin();
  NodePathMap::const_iterator end = nodeMap.end();

  os << endl;
  while (it != end)
  {
    os << it->first << "\t";
    os << it->second.distance << " ";
    os << it->second.marked;
    os << "\tpath :" << it->second.path << endl;
    it++;
  }
  os << "-------------------------\n";
}

} // namespace graphlib

/* end of file */
