/*  eigen.h - Teil aus CCMATH mathematics library source code.
 *
 *  Copyright (C)  2000   Daniel A. Atkinson    All rights reserved.
 *  This code may be redistributed under the terms of the GNU library
 *  public license (LGPL). ( See the lgpl.license file for details.)
 * ------------------------------------------------------------------------
 */



#ifndef EIGEN_H
#define EIGEN_H

namespace graphlib {

/**
  * @short Compute the eigenvalues and eigenvectors of a real symmetric
  * matrix A.
  *
  * The input and output matrices are related by
  * <p>
  *   A = E*D*E~ where D is the diagonal matrix of eigenvalues
  *   D[i,j] = ev[i] if i=j and 0 otherwise.
  * <p>
  * The columns of E are the eigenvectors.
  *
  * @param  a pointer to store for symmetric n by n input
            matrix A. The computation overloads this with an
            orthogonal matrix of eigenvectors E.
  * @param  ev pointer to the array of the output eigenvalues
  * @param  n dimension parameter (dim(a)= n*n, dim(ev)= n)
  */
void eigen(double *a,double *ev,int n);

/**
  * @short Transform a real symmetric matrix to tridiagonal form and
  *  compute the orthogonal matrix of this transformation.
  * The orthogonal transformation matrix O satisfies  O~*T*O = A.
  *
  * @param a pointer to array of symmetric input matrix A
  *          The computation overloads this array with the
  *          orthogonal transformation matrix O.
  * @param d pointer to array of diagonal output elements
  * @param dp pointer to array of n-1 elements neighboring the
  *            diagonal in the symmetric transformed matrix
  * @param n dimension (dim(a)= n*n, dim(d)=dim(dp)=n)
  */
void housev(double *a,double *d,double *dp,int n);

/**
  * @short Perform a QR reduction of a real symmetric tridiagonal matrix
  *   to diagonal form and update an orthogonal transformation matrix.
  *
  * This function operates on the output of 'housev'.
  *
  * @param ev pointer to array of input diagonal elements
  *        that the computation overloads with eigenvalues
  * @param evec pointer array of orthogonal input matrix
  *        This is updated by the computation to a matrix
  *        of eigenvectors.
  * @param dp pointer to array input elements neighboring the
  *        diagonal. This array is altered by the computation.
  * @param n dimension (dim(ev)=dim(dp)= n)
  */
int qrevec(double *ev,double *evec,double *dp,int n);

/**
  * @short Transpose a real square matrix in place A -> A~.
  *
  * @param a pointer to array of n by n input matrix A
  *        This is overloaded by the transpose of A.
  * @parma n dimension (dim(a)=n*n)
  */
void trnm(double *a,int n);

} // namespace graphlib
#endif // EIGEN_H

/* end of file */
