/***************************************************************************
                          eigenpair.h  -  description
                             -------------------
    begin                : Wed Mar 14 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef EIGENPAIR_H
#define EIGENPAIR_H

#include <iostream>
#include <map>
#include <list>
#include <vector>

namespace graphlib {

class EigenPair;
class Node;
typedef list<EigenPair*>::const_iterator constEigenPairIter;

/**
  * @short Eigenvalue and its eigenvectors
  *
  * @see Eigenvector_Eigenvalue
  */

class EigenPair
{
  	//------------------------------------------------------ friend functions
    /**
      * Output-operator.
      *
      * @param os the output-stream
      * @param eP EigenPair
      * @return the output-stream
      */
    friend ostream& operator<<(ostream& os,const EigenPair& eP);

  public:
  	//------------------------------------------------------ Con-/Destructors
    /**
      * Constructor. Creates a new EigenPair.
      *
      * @param size: dimension of the eigenvector
      */
    EigenPair(unsigned size);

    /**
      * Constructor. Creates a new EigenPair with <code>eigenvalue</code>,
      * <code>eigenvector</code> and <code>dimension</code>.
      *
      * @param eigenvalue
      * @param pointer to a matrix of eigenvector
      * @param dimension (number of columns / lines)
      * @param column of the matrix == eigenvector
      */
    EigenPair(double eigenvalue, double* eigenmatrix, unsigned dim, unsigned column);

    /**
      * Constructor. Creates a new EigenPair - the linear combination of all
      * EigenPairs from <code>eigenPairList</code> with <code>factors</code>
      * as multiplier.
      *
      * @param list of eigenpairs
      * @param factors
      */
    EigenPair(const list<EigenPair*>& eigenPairList, const vector<double>& factors);

    /**
      * Destructor. Deletes the EigenPair and frees the memory;
      */
    ~EigenPair();

  	//------------------------------------------------------ Operators
  	/**
  	  * Assignement Operator. Creates a new EigenPair with the same
  	  * eigenvalue and eigenvector as <code>ep</code>.
  	  */
  	EigenPair& operator= (const EigenPair& ep);
  	
  	/**
  	  * Returns true if <code>ep</code> has the same eigenvalues and
  	  * eigenvectors.
  	  */
  	bool operator== (const EigenPair& ep) const;
  	
  	/**
  	  * Returns true if the eigenvalue of <code>ep</code> is greater.
  	  */
  	bool operator< (const EigenPair& ep) const;
  	
  	//------------------------------------------------------ Information
  	/**
  	  * Returns the eigenvalue.
  	  *
  	  * @return eigenvalue
  	  */
  	double eigenValue() const { return eigenvalue; }
  	
  	/**
  	  * Returns the eigenvector.
  	  *
  	  * @return pointer to the eigenvector.
  	  */
  	double* eigenVector() const { return eigenvector; }
  	
  	/**
  	  * Returns the dimension of the eigenvector.
  	  *
  	  * @return dimension of eigenvector
  	  */
  	unsigned dimension() const { return dim; }
  	
  	/**
  	  * Returns the maximal value of the eigenvector.
  	  *
  	  * @return maximal value of eigenvector
  	  */
  	double maximumEV() const { return maximum; }
  	
  	/**
  	  * Returns the minimal value of the eigenvector.
  	  *
  	  * @return minimal value of eigenvector
  	  */
  	double minimumEV() const { return minimum; }
  	
  	//------------------------------------------------------ calculations
  	/**
  	  * Calculates the number of weak nodal domains.
  	  *
  	  * @param the list of nodes
  	  * @return number of weak nodal domains.
  	  */
  	int calcWeakNodalDomain(const list<Node>&);
  	
  	/**
  	  * Calculates the number of strong nodal domains.
  	  *
  	  * @param the list of nodes
  	  * @return number of strong nodal domains.
  	  */
  	int calcStrongNodalDomain(const list<Node>&);
  	
  	
  private:  	
  	//------------------------------------------------------ calculations
  	/// for calculating nodal domains
    map<Node,int>* makeNodeEigenVectorMap(const list<Node>& nodes);

  	//------------------------------------------------------ datas
    /// eigenvalue
    double      eigenvalue;
    /// eigenvector of eigenvalue
    double      *eigenvector;
    /// dimension of eigenvector
    unsigned    dim;
    /// lowest value of eigenvector
    double      minimum;
    /// hightest value of eigenvector
    double      maximum;
    /// number of weak nodal domains
    int         weakND;
    /// number of strong nodal domains
    int         strongND;
    /// map node <-> value of eigenvector
    map<Node,double> *nEvMap;

}; //class EigenPair

} // namespace graphlib

#include <functional>

namespace graphlib {
/**
  *@internal
  *
  */

class lessEigenPairPointer : public less<EigenPair*>
{
  public:
    bool operator()(const EigenPair* a, const EigenPair* b) const
    {
      return *a < *b ;
    }
}; // class lessEigenPairPointer

} // namespace graphlib
#endif // EIGENPAIR_H

/* end of file */
