/***************************************************************************
                          graph.h  -  description
                             -------------------
    begin                : Thu Mar 1 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPH_H
#define GRAPH_H

#include "basisgraph.h"
#include "gml_parser.h"
#include "coordinates.h"

#include <string>
#include <vector>
#include <list>
#include <iostream>

namespace graphlib {

class Graph;

typedef vector<Node>::const_iterator          cVNodeIter;
typedef vector<Edge>::const_iterator          cVEdgeIter;
typedef list<Graph*>::iterator             GraphIter;
typedef list<Graph*>::const_iterator       constGraphIter;

typedef vector<int>                      Path;

class AdjacencyMatrix;
/**
  *@short An undirected graph
  *
  * A graph <var>G=(V,E)</var> consists of a set of nodes <var>V</var>
  * and a set of edges <var>E</var>, where every edge can be viewed as
  * an unordered pair of nodes <var>(s,t)</var> connecting source
  * <var>s</var> with target <var>t</var>.
  *
  * @see Edge
  * @see Node
  *
  */

class Graph : virtual public BasisGraph
{
//    //------------------------------------------------------ friends
//    /**
//      * Output operator.
//      * Creates an GML output of the <code>g</code>.
//      *
//      * @param os the output-stream
//      * @param g the graph
//      * @see <a href="http://www.infosun.fmi.uni-passau.de/Graphlet/
//      *       GML/index.html">The GML File Format</a>
//      * @return the output-stream
//      */
//   friend ostream& operator<< (ostream& os, const Graph& g);
//
   public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Constructor.
      * Generates an empty graph, without any nodes and any edges.
      */
    Graph();


    /**
      * Constructor.
      * Generates a graph with the single <code>node</code>.
      *
      * @param Node
      */
    Graph(const Node&);

  private:
    /**
      * CopyConstructor -> not implemented
      */
    Graph(const Graph& g);

  public:
    /**
      * Destructor. Deletes all nodes and edges.
      */
    virtual ~Graph();

    //------------------------------------------------------ Operators
    /**
      * Assignment Operator.
      * <em>Please note:</em> This will generate an
      * isomorpic copy of <code>g</code>. Althoug this graph will look like
      * <code>g</code> it is not <em>physically</em> the same. Especially
      * it consists of nodes and edges, which of course have counterparts
      * in <code>g</code>, but are different.
      *
      * @param Graph the original graph
      */
    Graph& operator=(const Graph& g);

    /**
      * Assignment Operator
      * Generates a graph from the <code>adjacency matrix</code>.
      *
      * @param adjacency matrix
      */
    Graph& operator=(const Matrix&);

    //------------------------------------------------------ Information  	
    /**
      * Returns the <code>name</code> of the graph.
      *
      * @return name
      */
    string name() const;

    /**
      * Sets the <code>name</code> of the graph.
      *  	
      * @param name
      */
    void name(string name);

    /**
      * If <code>x</code> or <code>y</code> not within
      * <code>minCoord</code> or <code>maxCoord</code>,
      * sets them new.
      *
      * @param x
      * @param y
      */
    void newMinMaxCoord(double min, double max);

    /**
      * Sets <code>minCoord</code> and <code>maxCoord</code>
      * to -inf and +inf.
      */
    void resetMinMaxCoord();

    //------------------------------------------------------ Creation
    /**
      * Adds a new node. Id of the node = next free one. (starting with 1)
      *
      * @param id of node
      * @return iterator to new node
      */
    NodeIter newNode();

    /**
      * Adds a new edge from <code>s</code> to <code>t</code>.
      * Increases degrees and adds to neighbours from <code>s</code>
      * and <code>t</code>.
      *
      * <p>
      * <em>Precondition:</em> <code>s,t</code> are valid nodes in the graph.
      *
      * @param s source of new edge
      * @param t target of new edge
      * @param label label of new edge (optional)
      *
      * @return iterator to new edge
      */
    EdgeIter newEdge(const Node& s, const Node& t, string label="");

    /**
      * Adds a new edge from <code>s</code> to <code>t</code> with
      * <code>label</code>.
      * Increases degrees and adds to neighbours from <code>s</code> and
      * <code>t</code>.
      * <code>label</code> is optional.
      *
      * <p>
      * <em>Precondition:</em> <code>s,t</code> are valid nodes in the graph.
      *
      * @param s source of new edge
      * @param t target of new edge
      * @param label label of new edge (optional)
      *
      * @return iterator to new edge
      */
    EdgeIter newEdge(int source, int target, string label="");

    /**
      * Generates a subgraph with all nodes form <code>begin</code>
      * to <code>end</code> and if <code>induced</code>  is true
      * (default value) althought its adjacent edges.
      * <em>Please note:</em> This will generate isomorpic copies
      * of nodes and edges.
      *
      * @param <code>begin</code> of a list of node
      * @param <code>end</code> of a list node
      * @param induced.
      *
      * @return pointer to itself.
      */
    Graph* makeSubgraph(constNodeIter begin, constNodeIter end, bool induced=true);

    /**
      * Generates a subgraph with all edges form <code>begin</code>
      * to <code>end</code>.
      * <em>Please note:</em> This will generate isomorpic copies
      * of nodes and edges.
      *
      * @param <code>begin</code> of a vector of edges
      * @param <code>end</code> of a vector edges
      *
      * @return pointer to itself.
      */
    Graph* makeSubgraph(cVEdgeIter begin, cVEdgeIter end);

    //------------------------------------------------------ Deletion
    /**
      * Deletes node <code>n</code>, and thus all edges incident with
      * <code>n</code>.
      * <p>
      * <em>Precondition:</em> <code>n</code> is a valid nodes in the graph,
      * else a logic_error will be thrown.
      *
      * @param n valid node to be deleted
      */
    void delNode(Node& n);

    /**
      * Deletes all nodes and edges of this graph.
      *
      * @see Graph#clear
      */
    void delAllNodes();

    /**
      * Deletes edge <code>e</code>.
      * <p>
      * <em>Precondition:</em> <code>e</code> is a valid edge in the graph,
      * else a logic_error will be thrown.
      *
      * @param <code>e</code> valid edge to be deleted
      */
    void delEdge(Edge& e);

    /**
      * Deletes all edges of this graph.
      */
    void delAllEdges();

    /**
      * Deletes all nodes and edges.
      */
    virtual void clear();

    //------------------------------------------------------ I/O
    /**
      * Load graph from a file in GML-format.
      *
      * @param fname file in GML-format.
      * @return detailed error description (hopefully GML_OK). For
      *   details see @ref GM_error#err_num.
      * @see
      *   <a href="http://www.infosun.fmi.uni-passau.de/Graphlet/GML/index.html">
      *   The GML File Format</a>
      */
    GML_error load(const string& filename);

    //------------------------------------------------------ Search
    /**
      * Calculates all directed pathes between <code>x</code> and
      * <code>r</code>.
      *
      * @param x start node
      * @param r end node
      * @param currentPath (for recursion)
      * @return all directed pathes
      */
    vector<Path> vectorPaths(int x, int r, Path& currentPath);

   protected:
    //------------------------------------------------------ Name
    /// name of the graph
    string        name_;
    //------------------------------------------------------ Flag
    /// list of free node numbers - first element = the next positive
    /// number, second element = the next negative number which could
    /// be assigned
    /// other elements = gaps between first and second element
    list<int> freeNodeNumbers;
    /// minimal coordinates
    Coordinates minCoords;
    /// maximal coordinate
    Coordinates maxCoords;

    //------------------------------------------------------ Utilities
    /// Needed for creating isomorphic copies of a graph.
    void copy(const Graph& g);

  private:
    //------------------------------------------------------ for copy
    /// Needed for creating isomorphic copies of a graph.
    void copyList(constNodeIter, constNodeIter);
    /// Needed for creating isomorphic copies of a graph.
    void copyList(constEdgeIter, constEdgeIter);
    /// Needed for creating isomorphic copies of a graph.
    void copyNode(const Node& n);
    /// Needed for creating isomorphic copies of a graph.
    void copyEdge(const Edge& e);
    //------------------------------------------------------ for clear
    /// Needed for deleting nodes
    void delList(list<Node>&);
    /// Needed for deleting edges
    void delList(list<Edge>&);

    //------------------------------------------------------ for creation
  private:
    /// doing the initializing things
    void init();
    /// Needed for creating new nodes
    int getFreeNodeNumber();
    /// Needed for creating new nodes
    bool getFreeNodeNumber(int);
    /// Needed for creating the list of free node numbers
    void setFreeNodeNumber();
    /// Needed for deleting nodes
    void setFreeNodeNumber(int);
    /// for reading graphs from a gml file
    GML_error loadNodes(GML_pair*, GML_stat*);
    /// for reading graphs from a gml file
    GML_error loadEdges(GML_pair*, GML_stat*);

};  // class Graph

} // namespace graphlib
#endif // GRAPH_H

/* end of file */
