/***************************************************************************
                          node.h  -  description
                             -------------------
    begin                : Thu Mar 1 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NODE_H
#define NODE_H

#include <string>
#include <list>

#include "edge.h"
#include "graphlib_error.h"
using namespace graphlibError;

namespace graphlib {

class Node;
typedef list<Node>::iterator        NodeIter;
typedef list<Node>::const_iterator  constNodeIter;

class NodeData;
class Coordinates;

/**
  *@short A node in a graph.
  *
  *@author Gleiss Petra Manuela
  */
class Node
{
    //------------------------------------------------------ friends
    friend ostream& operator<< (ostream&, const Node&);
    friend class Edge;
    friend class Graph;

  public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Default constructor. Creates an invalid node.
      * The only way to obtain a valid node is through
      * @ref Graph#newNode
      * Example:
      * <pre>
      * Graph g;
      * Node n;
      *
      * n = g.newNode();
      * </pre>
      *
      * @see Graph#newNode
      */
    Node();

    //------------------------------------------------------ Operators
    /**
      * Return true, when <code>n</code> has same data adress.
      */
    bool operator==(const Node &n) const;

    /**
      * Return true, when <code>n</code> has not same data adress.
      */
    bool operator!=(const Node &n) const;

    /**
      * Return true, when id of <code>n</code>-data is lower.
      */
    bool operator<(const Node &n) const;

    /**
      * Return true, when id of <code>n</code>-data is higher.
      */
    bool operator>(const Node &n) const;

    //------------------------------------------------------ Information

    /**
      * Returns the id of the node.
      */
    int id() const;

    /**
      *@internal (Returns the position in list of all nodes.)
      */
    constNodeIter pos() const;

    /**
      * Returns the degree of this node.
      */
    unsigned degree() const;

    /**
      * Returns the label of this node.
      */
    string label() const;

     /**
       * Returns the Coordinates of this node
       */
    Coordinates coord() const;

     /**
       * Returns the node on the opposite side of <code>e</code>.
       *
       *@param <code>e</code> an edge incident ot the node
       */
    const Node& opposite(const Edge& e) const;

     /**
       * @return true when data == NULL
       */
    bool isNULL() const;

    /**
      * Returns true if there exists a directed
      * edge = {<code>source</code>, this}
      */
    bool isTarget(int source) const;

    /**
      * Returns the position in the list of all adjacent edges
      * of this node of the edge with
      * this node and <code>node</code> as endvertices.
      * If there's no such edge the end will be returned.
      *
      * @param id of a valid node
      * @return const_iterator to the edge or end
      */
    constEdgeIter findEdge(const Node& node) const;

    //------------------------------------------------------ Set functions
    /**
      * Sets <code>label</code> of the node
      *
      * @return const_iterator to the node
      */
    constNodeIter label(string label);

    /**
      * Sets <code>coordinates</code> of the node
      *
      * @return const_iterator to the node
      */
    constNodeIter coord(Coordinates coordinates);

    //------------------------------------------------------ Iterators
    /**
      * @return begin of all adjacent edges
      */
    constEdgeIter adjBegin() const;

    /**
      * @return end of all adjacent edges
      */
    constEdgeIter adjEnd() const;

  private:
    //------------------------------------------------------ datas
    /// datas
    NodeData *data_;

    //------------------------------------------------------ privat functions
    /// to get data
    const NodeData* data() const throw(EmptyNodeError);
    NodeData* data() throw(EmptyNodeError);


}; // class Node


} // namespace graphlib
#endif // NODE_H

/* end of file */
