/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Sat Jul 28 13:04:37 CEST 2001
    copyright            : (C) 2001 by Petra Gleiss
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <cstdlib>
#include <unistd.h>


#include <graphlib/cyclegraph.h>
#include <graphlib/graphlib_error.h>

using namespace graphlib;


#include <graphlib/logfile.h>
#include <graphlib/trace.h>

// logging exceptions and tracing
graphlib::Trace trace;

// logging debug messages
graphlib::LogFile logfile;

// commandline option help
static void usage(char *program);


int main(int argc, char *argv[])
{
  // initialize tracefile
  INIT_TRACE("trace.cycdeco")

  // initialize logfile
  INIT_LOGFILE("log.cycdeco")

  CycleGraph *graph;
  string filename;
  string unodefile;
  ifstream liste;

  // variables for the commandline
  int options;
  bool fileliste = false;
  bool statistic = false;
  bool foreach   = true;
  bool unode     = false;
  DecompositionsMode flag = All;
  bool again = false;
  int optNumb = 1;

  // no filename is given
  if (argc == 1)
    usage(argv[0]);


  try
  {
    // parse command line options
    while ((options = getopt(argc, argv, "lmrusqh?U:")) != EOF)
    {
      switch (options)
      {
      case 'l':
        fileliste=true;
      break;

      case 'm':
        flag = Horton;
        break;

      case 'r':
        flag = Relevants;
        break;

      case 'u':
        flag = UniqueShortests;
        break;

      case 's':
        statistic = true;
        break;

      case 'q':
        foreach = false;
        break;

      case 'U':
        unodefile = optarg;
        unode = true;
        optNumb++;
        break;

      case 'h':
      case '?':
      default: usage(argv[0]);
      }
      optNumb++;
    }

    if (argc == optNumb)
      usage(argv[0]);


    // open the list of file == last argument in commandline
    if (fileliste)
    {
      liste.open(argv[argc-1],ios::in);
      if (liste.fail() || liste.bad())
      {
        cerr << "Can not open file-liste: " << argv[argc-1] << endl;
        exit(1);
      }
    }

    do
    {
      // not again;
      again = false;

      // filename == last argument in commandline
      if (!fileliste)
        filename = argv[argc-1];

      else
      {
        if (liste.eof())
          return 0;

        // get filename
        getline(liste,filename);

        // file liste is not at the end
        again = true;

        if (filename == "")
          return 0;
      }

      // new CycleGraph
      graph = new CycleGraph();

      try
      {
        // read gml.file
        GML_error err = graph->load(filename);

        // error in the gml file
        if (err.err_num != GML_OK)
          THROW(FileError,"*** GML Error "+GML_error2string(err))

        if (unode)
        {
          // sets U-nodes
          graph->setUnodes(unodefile);
          // calcuate cycles and u-paths
          graph->calcUBasis(flag);
        }
        else {
          // write graph to logfile
          LOG1(Ttos(*graph));

          // calculate cycles
          graph->calcCycleBasis(flag);
        }

        // write cycles to file
        if (foreach)
        {
          string outputName = filename + ".cycles";
          ofstream out(outputName.c_str());
          out << graph->relevantCycles() << endl;
        }

        // make cycle statistic
        if (statistic)
         graph->makeCycleStatistic(filename);
      }
      catch(FileError &e)
      {
        cerr << "Wrong gml format in " << filename << "!\n" << e.what() << endl;
      }
      catch(exception &e)
      {
        cerr << "Error with graph in " << filename << "!\n" << e.what() << endl;
      }

      // delete graph
      delete graph;

    } while (again);
  }
  catch(exception &e)
  {
    cerr << "*** error ***\n";
    cerr << e.what() << endl;
    exit(-1);
  }
  return 0;
}


// help for the command line options
static void usage(char *program)
{
  cout << endl;
  cout << "\nUsage: " << program;
  cout << " [-l] [-U UNODE_FILE] [-m|r|u] [-q] [-s] [-h|?] FILE\n\n";
  cout << "  calculates the following cycle-types from the graph in FILE:\n";
  cout << "  relevant, shortest, unique shortest and essential cycles,\n";
  cout << "  an minimum cycle basis and the interchangeablility classes.\n\n";
  cout << "  If a UNODE_FILE is given, cycles and u-pathes are calculated.\n\n";
  cout << "  The cycles and paths are written on FILE.cycles\n\n";

  cout << "\t-l \t\tFILE is a file with a list of graphfiles\n";
  cout << "\t-q \t\tno cycle output\n";
  cout << "\t-s \t\tstatistic of cycles is written on FILE.cycle_stat\n\n";
  cout << "\t-m \t\tonly minimum cycle basis are calculated.\n";
  cout << "\t-r \t\tonly relevant cycles are calculated.\n";
  cout << "\t-u \t\tonly relevant and (unique) shortest cycles are calculated.\n\n";
  cout << "\t-U [UNODE_FILE]\tUNODE_FILE contains id of U-nodes separated with spaces.\n\n";
  cout << "\t-h, -?\tthis message\n";
  cout << endl;
  exit (42);
}

/* end of file */
